@@ -1,4 +1,4 @@
-#include "../../utils/cstring_t.h"
+#include <string>
 #include "../../tpl/slist_tpl.h"
 #include "../bildliste2d_besch.h"
 #include "obj_node.h"
@@ -6,15 +6,15 @@
 #include "imagelist2d_writer.h"
 
 
-void imagelist2d_writer_t::write_obj(FILE* fp, obj_node_t& parent, const slist_tpl<slist_tpl<cstring_t> >& keys)
+void imagelist2d_writer_t::write_obj(FILE* fp, obj_node_t& parent, const slist_tpl<slist_tpl<std::string> >& keys)
 {
 	bildliste2d_besch_t besch;
 
 	obj_node_t node(this, 4, &parent);
 
-	slist_iterator_tpl<slist_tpl<cstring_t> > iter(keys);
+	slist_iterator_tpl<slist_tpl<std::string> > iter(keys);
 
-	besch.anzahl = keys.count();
+	besch.anzahl = keys.get_count();
 
 	while (iter.next()) {
 		imagelist_writer_t::instance()->write_obj(fp, node, iter.get_current());
