@@ -12,7 +12,6 @@
 #include "../halthandle_t.h"
 #include "../simimg.h"
 #include "../simcolor.h"
-#include "../simdepot.h"
 #include "../dataobj/koord3d.h"
 #include "../dataobj/dingliste.h"
 #include "wege/weg.h"
@@ -21,7 +20,6 @@
 class spieler_t;
 class depot_t;
 class karte_t;
-class grund_info_t;
 class cbuffer_t;
 
 
@@ -69,9 +67,15 @@ template<> struct map_ding<stadtauto_t>   { static const ding_t::typ code = ding
 template<> struct map_ding<automobil_t>   { static const ding_t::typ code = ding_t::automobil;   };
 template<> struct map_ding<tunnel_t>      { static const ding_t::typ code = ding_t::tunnel;      };
 template<> struct map_ding<wayobj_t>      { static const ding_t::typ code = ding_t::wayobj;      };
+template<> struct map_ding<weg_t>         { static const ding_t::typ code = ding_t::way;         };
 template<> struct map_ding<zeiger_t>      { static const ding_t::typ code = ding_t::zeiger;      };
 
 
+template<typename T> static inline T* ding_cast(ding_t* const d)
+{
+	return d->get_typ() == map_ding<T>::code ? static_cast<T*>(d) : 0;
+}
+
 
 /**
  * <p>Abstrakte Basisklasse fr Untergrnde in Simutrans.</p>
@@ -114,10 +118,16 @@ public:
 	 */
 	static bool show_grid;
 
-	/* true, when only underground should be visible
-	 * @author kierongreen
-	 */
-	static bool underground_mode;
+	/* underground modes */
+	/* @author Dwachs    */
+	enum _underground_modes {
+		ugm_none = 0,	// normal view
+		ugm_all  = 1,   // everything underground visible, grid for grounds
+		ugm_level= 2	// overground things visible if their height  <= underground_level
+						// underground things visible if their height == underground_level
+	};
+	static uint8 underground_mode;
+	static sint8 underground_level;
 
 protected:
 	/**
@@ -178,7 +188,7 @@ public:
 	 * setzt die Bildnr. des anzuzeigenden Bodens
 	 * @author Hj. Malthaner
 	 */
-	inline void setze_bild(image_id n)
+	inline void set_bild(image_id n)
 	{
 		bild_nr = n;
 		set_flag(dirty);
@@ -213,9 +223,9 @@ public:
 	static void toggle_grid() { grund_t::show_grid = !grund_t::show_grid; }
 
 	/**
-	 * Toggle underground display (now only a flag)
+	 * Sets the undergroundmode & level
 	 */
-	static void toggle_underground_mode() { grund_t::underground_mode = !grund_t::underground_mode; }
+	static void set_underground_mode(const uint8 ugm, const sint8 level);
 
 	karte_t *get_welt() const {return welt;}
 
@@ -223,12 +233,10 @@ public:
 	* Setzt Flags fr das neuzeichnen genderter Untergrnde
 	* @author Hj. Malthaner
 	*/
-	inline void set_flag(enum flag_values flag) {flags |= flag;}
+	inline void set_flag(flag_values flag) {flags |= flag;}
 
-	inline void clear_flag(enum flag_values flag) {flags &= ~flag;}
-	inline bool get_flag(enum flag_values flag) const {return (flags & flag) != 0;}
-
-	void entferne_grund_info() const;
+	inline void clear_flag(flag_values flag) {flags &= ~flag;}
+	inline bool get_flag(flag_values flag) const {return (flags & flag) != 0;}
 
 	/**
 	* start a new month (and toggle the seasons)
@@ -248,35 +256,40 @@ public:
 	* @return Die Nummer des Bildes des Untergrundes.
 	* @author Hj. Malthaner
 	*/
-	inline image_id gib_bild() const {return bild_nr;}
+	inline image_id get_bild() const {return bild_nr;}
 
 	/**
 	* Returns the number of an eventual foundation
 	* @author prissi
 	*/
-	image_id gib_back_bild(int leftback) const;
+	image_id get_back_bild(int leftback) const;
 	virtual void clear_back_bild() {back_bild_nr=0;}
 
+	/**
+	* if ground is deleted mark the old spot as dirty
+	*/
+	void mark_image_dirty();
+
 	/**
 	* Gibt den Namen des Untergrundes zurueck.
 	* @return Den Namen des Untergrundes.
 	* @author Hj. Malthaner
 	*/
-	virtual const char* gib_name() const = 0;
+	virtual const char* get_name() const = 0;
 
 	/**
 	* Gibt den Typ des Untergrundes zurueck.
 	* @return Der Typ des Untergrundes.
 	* @author Hj. Malthaner
 	*/
-	virtual enum grund_t::typ gib_typ() const {return grund;}
+	virtual typ get_typ() const { return grund; }
 
 	/**
 	* Gibt eine Beschreibung des Untergrundes (informell) zurueck.
 	* @return Einen Beschreibungstext zum Untergrund.
 	* @author Hj. Malthaner
 	*/
-	const char* gib_text() const;
+	const char* get_text() const;
 
 	/**
 	* @return NULL
@@ -289,7 +302,7 @@ public:
 	* Oeffnet standardmaessig kein Infofenster.
 	* @author Hj. Malthaner
 	*/
-	bool zeige_info();
+	void zeige_info();
 
 	/**
 	* Gibt die Farbe des Beschreibungstexthintergrundes zuurck
@@ -302,7 +315,7 @@ public:
 	 * Sets the label text (by copying it)
 	 * @param text the new text (NULL will remove any label text)
 	 */
-	void setze_text(const char* text);
+	void set_text(const char* text);
 
 	virtual bool ist_natur() const {return false;}
 	virtual bool ist_wasser() const {return false;}
@@ -312,21 +325,23 @@ public:
 	* cannot be virtual - subclasses must set the flags appropriately!
 	* @author Hj. Malthaner
 	*/
-	inline bool ist_bruecke() const {return gib_typ()==brueckenboden;}
+	inline bool ist_bruecke() const {return get_typ()==brueckenboden;}
 
 	/**
-	* This is called very often, it must be inlined and therefore
-	* cannot be virtual - subclasses must set the flags appropriately!
-	* @author Hj. Malthaner
+	* true if tunnelboden (hence true also for tunnel mouths)
+	* check for visibility in is_visible()
 	*/
-	inline bool ist_tunnel() const {return ((gib_typ()==tunnelboden)^grund_t::underground_mode);}
+	inline bool ist_tunnel() const {
+		return ( (get_typ()==tunnelboden) );
+	}
 
 	/**
-	* This is called very often, it must be inlined and therefore
-	* cannot be virtual - subclasses must set the flags appropriately!
-	* @author Hj. Malthaner
+	* gives true for grounds inside tunnel (not tunnel mouths)
+	* check for visibility in is_visible()
 	*/
-	inline bool ist_im_tunnel() const {return (gib_typ()==tunnelboden  &&  ist_karten_boden()==0)^grund_t::underground_mode;}
+	inline bool ist_im_tunnel() const {
+		return ( get_typ()==tunnelboden && (!ist_karten_boden())) ;
+	}
 
 	/* this will be stored locally, since it is called many, many times */
 	inline uint8 ist_karten_boden() const {return (flags&is_kartenboden);}
@@ -336,7 +351,7 @@ public:
 	* returns powerline here
 	* @author Kieron Green
 	*/
-	leitung_t *gib_leitung() const { return (leitung_t *) dinge.get_leitung(); }
+	leitung_t *get_leitung() const { return (leitung_t *) dinge.get_leitung(); }
 
 	/**
 	* Laedt oder speichert die Daten des Untergrundes in eine Datei.
@@ -354,47 +369,136 @@ public:
 	* @return Die Position des Grundes in der 3d-Welt
 	* @author Hj. Malthaner
 	*/
-	inline const koord3d& gib_pos() const { return pos; }
+	inline const koord3d& get_pos() const { return pos; }
 
-	inline void setze_pos(koord3d newpos) { pos = newpos;}
+	inline void set_pos(koord3d newpos) { pos = newpos;}
 
 	// slope are now maintained locally
-	hang_t::typ gib_grund_hang() const { return (hang_t::typ)slope; }
-	void setze_grund_hang(hang_t::typ sl) { slope = sl; }
+	hang_t::typ get_grund_hang() const { return (hang_t::typ)slope; }
+	void set_grund_hang(hang_t::typ sl) { slope = sl; }
 
 	/**
 	 * Manche Bden knnen zu Haltestellen gehren.
 	 * @author Hj. Malthaner
 	 */
-	void setze_halt(halthandle_t halt);
+	void set_halt(halthandle_t halt);
 
 	/**
 	 * Ermittelt, ob dieser Boden zu einer Haltestelle gehrt.
 	 * @return NULL wenn keine Haltestelle, sonst Zeiger auf Haltestelle
 	 * @author Hj. Malthaner
 	 */
-	halthandle_t gib_halt() const;
+	halthandle_t get_halt() const;
 	bool is_halt() const { return flags & is_halt_flag; }
 
-	inline sint16 gib_hoehe() const {return pos.z;}
+	inline sint8 get_hoehe() const {return pos.z;}
 
-	void setze_hoehe(int h) { pos.z = h;}
+	void set_hoehe(int h) { pos.z = h;}
 
+	// Helper functions for underground modes
+	//
+	// returns the height for the use in underground-mode,
+	// heights above underground_level are cutted
+	inline sint8 get_disp_height() const {
+		return (underground_mode & ugm_level )
+			? (pos.z > underground_level ? underground_level : pos.z)
+			: pos.z ;
+	}
+
+	// returns slope
+	// if tile is not visible, 'flat' is returned
+	// special care has to be taken of tunnel mouths
+	inline hang_t::typ get_disp_slope() const {
+		return (  (underground_mode & ugm_level)  &&  (pos.z > underground_level  ||  (get_typ()==tunnelboden  &&  ist_karten_boden()  &&  pos.z == underground_level))
+							? (hang_t::typ)hang_t::flach
+							: get_grund_hang() );
+
+		/*switch(underground_mode) {// long version of the return statement above
+			case ugm_none: return(get_grund_hang());
+			case ugm_all:  return(get_grund_hang()); // get_typ()==tunnelboden && !ist_karten? hang_t::flach : get_grund_hang());
+			case ugm_level:return((pos.z > underground_level || (get_typ()==tunnelboden && ist_karten_boden() && pos.z == underground_level))
+							? hang_t::flach
+							: get_grund_hang());
+		}*/
+	}
+
+	inline bool is_visible() const {
+		if(get_typ()==tunnelboden) {
+			switch(underground_mode) {
+				case ugm_none: return ist_karten_boden();
+				case ugm_all:  return true;
+				case ugm_level:return  pos.z == underground_level  ||  (ist_karten_boden()  &&  pos.z <= underground_level);
+			}
+		}
+		else {
+			switch(underground_mode) {
+				case ugm_none: return true;
+				case ugm_all:  return false;
+				case ugm_level:return pos.z <= underground_level;
+			}
+		}
+		return(false);
+	}
+
+	// the same as above but specialized for kartenboden
+	inline bool is_karten_boden_visible() const {
+		switch(underground_mode) {
+			case ugm_none: return true;
+			case ugm_all:  return get_typ()==tunnelboden;
+			case ugm_level:return pos.z <= underground_level;
+		}
+		return(false);
+	}
 	/**
-	* Zeichnet Bodenbild des Grundes
+	 * returns slope of ways as displayed (special cases: bridge ramps, tunnel mouths, undergroundmode etc)
+	 */
+	hang_t::typ get_disp_way_slope() const;
+	/**
+	* displays the ground images (including foundations, fences and ways)
 	* @author Hj. Malthaner
 	*/
-	void display_boden(const sint16 xpos, sint16 ypos) const;
+	void display_boden(const sint16 xpos, const sint16 ypos, const sint16 raster_tile_width) const;
 
-	/* displays everything that is on a tile;
+	void display_if_visible(sint16 xpos, sint16 ypos, sint16 raster_tile_width) const;
+
+	/**
+	 * displays everything that is on a tile - the main display routine for objects on tiles
 	 * @param is_global set to true, if this is called during the whole screen update
+	 * @author dwachs
 	 */
-	void display_dinge(const sint16 xpos, sint16 ypos, const bool called_from_simview) const;
+	void display_dinge_all(const sint16 xpos, const sint16 ypos, const sint16 raster_tile_width, const bool is_global) const;
 
+	/**
+	 * displays background images of all non-moving objects on the tile
+	 * @param is_global set to true, if this is called during the whole screen update
+	 * @param draw_ways if true then draw images of ways
+	 * @param visible if false then draw only grids and markers
+	 * @return index of first vehicle on the tile
+	 * @author dwachs
+	 */
+	uint8 display_dinge_bg(const sint16 xpos, const sint16 ypos, const bool is_global, const bool draw_ways, const bool visible) const;
+
+	/**
+	 * displays vehicle (background) images
+	 * @param is_global set to true, if this is called during the whole screen update
+	 * @param start_offset start with object at this index
+	 * @param ribi draws only vehicles driving in this direction (or against this)
+	 * @param ontile is true if we are on the tile that defines the clipping
+	 * @author dwachs
+	 */
+	uint8 display_dinge_vh(const sint16 xpos, const sint16 ypos, const bool is_global, const uint8 start_offset, const ribi_t::ribi ribi, const bool ontile) const;
+
+	/**
+	 *  displays all foreground images
+	 * @param is_global set to true, if this is called during the whole screen update
+	 * @author dwachs
+	 */
+	void display_dinge_fg(const sint16 xpos, const sint16 ypos, const bool is_global, const uint8 start_offset) const;
 	/* overlayer with signs, good levels and station coverage
 	 * resets the dirty flag
 	 * @author kierongreen
 	 */
+
 	void display_overlay(sint16 xpos, sint16 ypos);
 
 	inline ding_t *first_obj() const { return dinge.bei(offsets[flags/has_way1]); }
@@ -408,14 +512,14 @@ public:
 	bool obj_loesche_alle(spieler_t *sp) { return dinge.loesche_alle(sp,offsets[flags/has_way1]); }
 	bool obj_ist_da(const ding_t* obj) const { return dinge.ist_da(obj); }
 	ding_t * obj_bei(uint8 n) const { return dinge.bei(n); }
-	uint8  obj_count() const { return dinge.gib_top()-offsets[flags/has_way1]; }
-	uint8 gib_top() const {return dinge.gib_top();}
+	uint8  obj_count() const { return dinge.get_top()-offsets[flags/has_way1]; }
+	uint8 get_top() const {return dinge.get_top();}
 
 	// moves all object from the old to the new grund_t
 	void take_obj_from( grund_t *gr);
 
 	/**
-	* @returns NULL wenn OK, oder Meldung, warum nicht
+	* @return NULL wenn OK, oder Meldung, warum nicht
 	* @author Hj. Malthaner
 	*/
 	const char * kann_alle_obj_entfernen(const spieler_t *sp) const { return dinge.kann_alle_entfernen(sp,offsets[flags/has_way1]); }
@@ -429,7 +533,7 @@ public:
 	* Falls es hier ein Depot gibt, dieses zurueckliefern
 	* @author Volker Meyer
 	*/
-	depot_t *gib_depot() const { return dynamic_cast<depot_t *>(first_obj()); }
+	depot_t *get_depot() const;
 
 	/*
 	* Interface zur Abfrage der Wege
@@ -442,23 +546,21 @@ public:
 	* The only way to get the typ of a way on a tile
 	* @author Hj. Malthaner
 	*/
-	weg_t *gib_weg_nr(int i) const { return (flags&(has_way1<<i)) ? static_cast<weg_t *>(obj_bei(i)) : NULL; }
+	weg_t *get_weg_nr(int i) const { return (flags&(has_way1<<i)) ? static_cast<weg_t *>(obj_bei(i)) : NULL; }
 
 	/**
 	* Inline da sehr oft aufgerufen.
 	* Sucht einen Weg vom typ 'typ' auf diesem Untergrund.
 	* @author Hj. Malthaner
 	*/
-	weg_t *gib_weg(waytype_t typ) const {
-		if(flags&has_way1) {
-			weg_t *w=(weg_t *)obj_bei(0);
-			if(w->gib_waytype()==typ) {
+	weg_t *get_weg(waytype_t typ) const {
+		if (weg_t* const w = get_weg_nr(0)) {
+			if(w->get_waytype()==typ) {
 				return w;
 			}
 		}
-		if(flags&has_way2) {
-			weg_t *w=(weg_t *)obj_bei(1);
-			if(w->gib_waytype()==typ) {
+		if (weg_t* const w = get_weg_nr(1)) {
+			if(w->get_waytype()==typ) {
 				return w;
 			}
 		}
@@ -467,30 +569,30 @@ public:
 
 	uint8 has_two_ways() const { return flags&has_way2; }
 
-	bool hat_weg(waytype_t typ) const { return gib_weg(typ)!=NULL; }
+	bool hat_weg(waytype_t typ) const { return get_weg(typ)!=NULL; }
 
 	/**
 	* Returns the system type s_type of a way of type typ at this location
 	* Currently only needed for tramways or other different types of rails
 	*
 	* @author DarioK
-	* @see gib_weg
+	* @see get_weg
 	*/
-	uint8 gib_styp(waytype_t typ) const
+	uint8 get_styp(waytype_t typ) const
 	{
-		weg_t *weg = gib_weg(typ);
-		return (weg) ? weg->gib_besch()->gib_styp() : 0;
+		weg_t *weg = get_weg(typ);
+		return (weg) ? weg->get_besch()->get_styp() : 0;
 	}
 
 	/**
 	* Ermittelt die Richtungsbits furr den weg vom Typ 'typ'.
 	* Liefert 0 wenn kein weg des Typs vorhanden ist. Ein Weg kann ggf.
 	* auch 0 als Richtungsbits liefern, deshalb kann die Anwesenheit eines
-	* Wegs nicht hierurber, sondern mit gib_weg(), ermittelt werden.
+	* Wegs nicht hierurber, sondern mit get_weg(), ermittelt werden.
 	* Also beware of water, which always allows all directions ...thus virtual
 	* @author Hj. Malthaner
 	*/
-	virtual ribi_t::ribi gib_weg_ribi(waytype_t typ) const;
+	virtual ribi_t::ribi get_weg_ribi(waytype_t typ) const;
 
 	/**
 	* Ermittelt die Richtungsbits furr den weg vom Typ 'typ' unmaskiert.
@@ -499,7 +601,7 @@ public:
 	* @author Hj. Malthaner/V. Meyer
 	*
 	*/
-	virtual ribi_t::ribi gib_weg_ribi_unmasked(waytype_t typ) const;
+	virtual ribi_t::ribi get_weg_ribi_unmasked(waytype_t typ) const;
 
 	/**
 	* checks a ways on this ground tile and returns the highest speedlimit.
@@ -512,7 +614,7 @@ public:
 	* only used for bridges, which start at a slope
 	* @author V. Meyer
 	*/
-	virtual sint8 gib_weg_yoff() const { return 0; }
+	virtual sint8 get_weg_yoff() const { return 0; }
 
 	/**
 	* Hat der Boden mindestens ein weg_t-Objekt? Liefert false fr Wasser!
@@ -525,15 +627,33 @@ public:
 	* Strassenbahnschienen duerfen nicht als Kreuzung erkannt werden!
 	* @author V. Meyer, dariok
 	*/
-	inline bool ist_uebergang() const { return (flags&has_way2)!=0  &&  ((weg_t *)dinge.bei(1))->gib_besch()->gib_styp()!=7; }
+	inline bool ist_uebergang() const { return (flags&has_way2)!=0  &&  ((weg_t *)dinge.bei(1))->get_besch()->get_styp()!=7; }
 
-	virtual hang_t::typ gib_weg_hang() const { return gib_grund_hang(); }
+	/**
+	* returns the vehcile of a convoi (if there)
+	* @author V. Meyer
+	*/
+	ding_t *get_convoi_vehicle() const { return dinge.get_convoi_vehicle(); }
+
+	virtual hang_t::typ get_weg_hang() const { return get_grund_hang(); }
+
+	/*
+	 * Search a matching wayobj
+	 */
+	wayobj_t *get_wayobj( waytype_t wt ) const;
 
 	/**
 	* Interface zur Bauen der Wege
 	* =============================
 	*/
 
+	/**
+	 * remove trees and groundobjs on this tile
+	 * called before building way or powerline
+	 * @return costs
+	 */
+	sint64 remove_trees();
+
 	/**
 	 * Bauhilfsfunktion - ein neuer weg wird mit den vorgegebenen ribis
 	 * eingetragen und der Grund dem Erbauer zugeordnet.
@@ -544,7 +664,7 @@ public:
 	 *
 	 * @author V. Meyer
 	 */
-	long neuen_weg_bauen(weg_t *weg, ribi_t::ribi ribi, spieler_t *sp);
+	sint64 neuen_weg_bauen(weg_t *weg, ribi_t::ribi ribi, spieler_t *sp);
 
 	/**
 	 * Bauhilfsfunktion - die ribis eines vorhandenen weges werden erweitert
@@ -614,7 +734,7 @@ public:
 	 * @author: Volker Meyer
 	 * @date: 21.05.2003
 	 */
-	sint16 get_vmove(koord dir) const;
+	sint8 get_vmove(koord dir) const;
 
 	/* removes everything from a tile, including a halt but i.e. leave a
 	 * powerline ond other stuff
@@ -624,6 +744,7 @@ public:
 
 	void* operator new(size_t s);
 	void  operator delete(void* p, size_t s);
+
 };
 
 
