@@ -35,8 +35,7 @@ kanal_t::kanal_t(karte_t *welt) : weg_t (welt)
 
 
 
-void
-kanal_t::rdwr(loadsave_t *file)
+void kanal_t::rdwr(loadsave_t *file)
 {
 	weg_t::rdwr(file);
 
@@ -51,7 +50,7 @@ kanal_t::rdwr(loadsave_t *file)
 	}
 	else {
 		char bname[128];
-		file->rdwr_str(bname, 128);
+		file->rdwr_str(bname, lengthof(bname));
 
 		const weg_besch_t *besch = wegbauer_t::get_besch(bname);
 		int old_max_speed = get_max_speed();
