@@ -25,7 +25,7 @@ class schiene_t : public weg_t
 {
 protected:
 	/**
-	* Bound when this block was sucessfully reserved by the convoi
+	* Bound when this block was successfully reserved by the convoi
 	* @author prissi
 	*/
 	convoihandle_t reserved;
@@ -74,7 +74,7 @@ public:
 	* true, then this rail was reserved
 	* @author prissi
 	*/
-	bool reserve(convoihandle_t c);
+	bool reserve(convoihandle_t c, ribi_t::ribi dir);
 
 	/**
 	* releases previous reservation
@@ -107,6 +107,17 @@ public:
 	 * @author kierongreen
 	 */
 	virtual PLAYER_COLOR_VAL get_outline_colour() const { return (show_reservations  &&  reserved.is_bound()) ? TRANSPARENT75_FLAG | OUTLINE_FLAG | COL_RED : 0;}
+
+	/*
+	 * to show reservations if needed
+	 */
+	virtual PLAYER_COLOR_VAL get_outline_bild() const {return weg_t::get_bild();}
 };
 
+
+template<> inline schiene_t* ding_cast<schiene_t>(ding_t* const d)
+{
+	return dynamic_cast<schiene_t*>(d);
+}
+
 #endif
