@@ -82,7 +82,13 @@ public:
 	static void register_besch(kreuzung_besch_t *besch);
 	static bool alles_geladen() {return true; }
 
-	static const kreuzung_besch_t *get_crossing(const waytype_t ns, const waytype_t ow, uint32 way_0_speed, uint16 timeline_year_month);
+	/**
+	 * returns descriptor for crossing wrt timeline
+	 * tries to match max-speeds:
+	 * (1) find crossings with maxspeed close to requested maxspeed
+	 * (2) prefer crossings with maxspeed larger than requested
+	 */
+	static const kreuzung_besch_t *get_crossing(const waytype_t ns, const waytype_t ow, sint32 way_0_speed, sint32 way_1_speed, uint16 timeline_year_month);
 
 	// returns a new or an existing crossing_logic_t object
 	// new, of no matching crossings are next to it
