@@ -2,8 +2,10 @@
 #define KOORD3D_H
 
 #include <stdlib.h>
-#include "../simtypes.h"
 #include "koord.h"
+#include "ribi.h"
+#include "../simtypes.h"
+#include "../tpl/vector_tpl.h"
 
 
 /**
@@ -108,27 +110,36 @@ static inline koord3d operator - (const koord3d& a, const koord& b)
 }
 
 
-static inline int koord_distance(koord a, koord b)
+static inline uint32 koord_distance(koord3d a, koord b)
 {
 	return abs(a.x - b.x) + abs(a.y - b.y);
 }
 
 
-static inline int koord_distance(koord3d a, koord b)
+static inline uint32 koord_distance(koord a, koord3d b)
 {
 	return abs(a.x - b.x) + abs(a.y - b.y);
 }
 
 
-static inline int koord_distance(koord a, koord3d b)
+static inline uint32 koord_distance(koord3d a, koord3d b)
 {
 	return abs(a.x - b.x) + abs(a.y - b.y);
 }
 
+/*
+ * This class defines a vector_tpl<koord3d> with some
+ * helper functions
+ * @author Gerd Wachsmuth
+ */
 
-static inline int koord_distance(koord3d a, koord3d b)
-{
-	return abs(a.x - b.x) + abs(a.y - b.y);
-}
+class koord3d_vector_t : public vector_tpl< koord3d > {
+public:
+	// computes ribi at position i
+	ribi_t::ribi get_ribi( uint32 index ) const;
+	// computes ribi at position i only if distance to previous/next is not larger than 1
+	ribi_t::ribi get_short_ribi( uint32 index ) const;
+	void rotate90( sint16 );
+};
 
 #endif
