@@ -8,11 +8,12 @@
 #ifndef dataobj_umgebung_h
 #define dataobj_umgebung_h
 
+#include <string>
 #include "../simtypes.h"
 #include "../simconst.h"
+#include "../simcolor.h"
 #include "einstellungen.h"
 
-class cstring_t;
 
 /**
  * Diese Klasse bildet eine Abstraktion der Kommandozeilenparameter.
@@ -32,16 +33,36 @@ public:
 	//points to the current directory user for loading and saving
 	static const char *user_dir;
 
-	static cstring_t objfilename;
+	// version for which the savegames should be created
+	static const char *savegame_version_str;
 
-	// maximum number of handles
-	static uint16 max_convoihandles;
-	static uint16 max_linehandles;
-	static uint16 max_halthandles;
+	static std::string objfilename;
+
+	// true, if we are in networkmode
+	static bool networkmode;
+	static long server_frames_ahead;
+	static long additional_client_frames_behind;
+	static long network_frames_per_step;
+	// how often to synchronize
+	static uint32 server_sync_steps_between_checks;
+	static bool restore_UI;	// when true, restore the windows from a savegame
+
+	// if we are the server, we are at this port ...
+	static uint16 server;
+
+	static uint32 announce_server;
+	static sint32 announce_server_intervall;
+	static std::string server_name;
+	static std::string server_comment;
 
 	// scrollrichtung
 	static sint16 scroll_multi;
 
+	// messages with player name
+	static bool add_player_name_to_message;
+
+	static sint16 window_snap_distance;
+
 	/**
 	* tag-nacht wechsel zeigen ?
 	*
@@ -76,6 +97,7 @@ public:
 
 	/**
 	* Namen (Stdte, Haltestellen) anzeigen? (0 .. 3)
+	* lable type 4..7
 	*
 	* @author Hj. Malthaner
 	*/
@@ -140,10 +162,6 @@ public:
 	*/
 	static bool window_buttons_right;
 
-	/*
-	 * frame active window
-	 */
-	static bool window_frame_active;
 
 	/**
 	* Produce more debug info ?
@@ -165,13 +183,6 @@ public:
 	*/
 	static sint32 intercity_road_length;
 
-	/**
-	* Typ (Name) initiale Stadtverbindungen
-	*
-	* @author Hj. Malthaner
-	*/
-	static const char *intercity_road_type;
-
 	/**
 	 * Name of rivers; first the river with the lowest number
 	 * @author prissi
@@ -179,6 +190,17 @@ public:
 	static const char *river_type[10];
 	static uint8 river_types;
 
+	enum date_fmt {
+		DATE_FMT_SEASON   = 0,
+		DATE_FMT_MONTH    = 1,
+		DATE_FMT_JAPANESE = 2,
+		DATE_FMT_US       = 3,
+		DATE_FMT_GERMAN   = 4,
+		DATE_FMT_JAPANESE_NO_SEASON = 5,
+		DATE_FMT_US_NO_SEASON       = 6,
+		DATE_FMT_GERMAN_NO_SEASON   = 7
+	};
+
 	/**
 	* show month in date?
 	*
@@ -201,13 +223,22 @@ public:
 	// false to quit the programs
 	static bool quit_simutrans;
 
-	// new game start without tree
-	static bool no_tree;
-
 	// customize your tooltips
 	static bool show_tooltips;
 	static uint8 tooltip_color;
 	static uint8 tooltip_textcolor;
+	static uint32 tooltip_delay;
+	static uint32 tooltip_duration;
+
+	// limit width and height of menu toolbars
+	static uint8 toolbar_max_width;
+	static uint8 toolbar_max_height;
+
+	// color used for cursor overlay blending
+	static uint8 cursor_overlay_color;
+
+	// show error/info tooltips over the vehicles
+	static uint8 show_vehicle_states;
 
 	// fixed day/night view level
 	static sint8 daynight_level;
@@ -219,8 +250,20 @@ public:
 	static sint16 global_volume, midi_volume;
 	static bool mute_sound, mute_midi, shuffle_midi;
 
+	static bool left_to_right_graphs;
+
+	// how to highlight topped (untopped windows)
+	static bool window_frame_active;
+	static uint8 front_window_bar_color;
+	static uint8 front_window_text_color;
+	static uint8 bottom_window_bar_color;
+	static uint8 bottom_window_text_color;
+
 	static einstellungen_t default_einstellungen;
 
+	// init with default values
+	static void init();
+
 	// load/saving settings from file
 	static void rdwr(loadsave_t *file);
 };
