@@ -10,7 +10,7 @@
  * Hj. Malthaner
  */
 
-class bruecke_t : public ding_t
+class bruecke_t : public ding_no_info_t
 {
 private:
 	const bruecke_besch_t *besch;
@@ -26,7 +26,12 @@ public:
 	image_id get_after_bild() const {return besch->get_vordergrund(img, get_pos().z+Z_TILE_STEP*(img>=bruecke_besch_t::N_Start  &&  img<=bruecke_besch_t::W_Start) >= welt->get_snowline()); }
 
 	const char *get_name() const {return "Bruecke";}
-	enum ding_t::typ get_typ() const {return bruecke;}
+	typ get_typ() const { return bruecke; }
+
+	/**
+	 * waytype associated with this object
+	 */
+	waytype_t get_waytype() const { return besch ? besch->get_waytype() : invalid_wt; }
 
 	const bruecke_besch_t *get_besch() const { return besch; }
 
@@ -41,9 +46,12 @@ public:
 
 	void entferne(spieler_t *sp);
 
-	void zeige_info() {} // show no info
-
 	void rotate90();
+	/**
+	 * @return NULL wenn OK, ansonsten eine Fehlermeldung
+	 * @author Hj. Malthaner
+	 */
+	virtual const char *ist_entfernbar(const spieler_t *sp);
 };
 
 #endif
