@@ -5,13 +5,8 @@
 
 #include "components/gui_label.h"
 
-#include "../besch/baum_besch.h"
-#include "../utils/cbuffer_t.h"
-
-#include "../simwerkz.h"
-
-#define MAX_BUILD_TYPE (6)
 
+class baum_besch_t;
 class wkz_plant_tree_t;
 
 class baum_edit_frame_t : public extend_edit_gui_t
@@ -36,14 +31,14 @@ public:
 	* @return den nicht uebersetzten Namen der Komponente
 	* @author Hj. Malthaner
 	*/
-	const char* gib_name() const { return "baum builder"; }
+	const char* get_name() const { return "baum builder"; }
 
 	/**
 	* Manche Fenster haben einen Hilfetext assoziiert.
 	* @return den Dateinamen fr die Hilfe, oder NULL
 	* @author Hj. Malthaner
 	*/
-	const char* gib_hilfe_datei() const { return "baum_build.txt"; }
+	const char* get_hilfe_datei() const { return "baum_build.txt"; }
 
 	/**
 	* This method is called if an action is triggered
