@@ -5,13 +5,8 @@
 
 #include "components/gui_label.h"
 
-#include "../besch/haus_besch.h"
-#include "../besch/fabrik_besch.h"
-#include "../utils/cbuffer_t.h"
-
-#include "../simwerkz.h"
-
-#define MAX_BUILD_TYPE (6)
+class haus_besch_t;
+class wkz_build_haus_t;
 
 
 class citybuilding_edit_frame_t : public extend_edit_gui_t
