@@ -19,6 +19,7 @@
 
 #include "../dataobj/fahrplan.h"
 #include "../dataobj/translator.h"
+#include "../dataobj/loadsave.h"
 #include "fahrplan_gui.h"
 // @author hsiegeln
 #include "../simlinemgmt.h"
@@ -26,18 +27,26 @@
 #include "../boden/grund.h"
 #include "messagebox.h"
 
+#include "../player/simplay.h"
+
 #include "../utils/simstring.h"
+#include "../utils/cbuffer_t.h"
 
 #include "components/gui_chart.h"
 #include "components/list_button.h"
 
 
+karte_t *convoi_detail_t::welt = NULL;
+
+
+
 convoi_detail_t::convoi_detail_t(convoihandle_t cnv)
 : gui_frame_t(cnv->get_name(), cnv->get_besitzer()),
   scrolly(&veh_info),
   veh_info(cnv)
 {
 	this->cnv = cnv;
+	welt = cnv->get_welt();
 
 	sale_button.init(button_t::roundbox, "verkaufen", koord(BUTTON4_X, 14), koord(BUTTON_WIDTH,BUTTON_HEIGHT));
 	sale_button.add_listener(this);
@@ -63,18 +72,10 @@ convoi_detail_t::convoi_detail_t(convoihandle_t cnv)
 }
 
 
-
-/**
- * komponente neu zeichnen. Die bergebenen Werte beziehen sich auf
- * das Fenster, d.h. es sind die Bildschirkoordinaten des Fensters
- * in dem die Komponente dargestellt wird.
- * @author Hj. Malthaner
- */
-void
-convoi_detail_t::zeichnen(koord pos, koord gr)
+void convoi_detail_t::zeichnen(koord pos, koord gr)
 {
 	if(!cnv.is_bound()) {
-		destroy_win(dynamic_cast <gui_fenster_t *>(this));
+		destroy_win(this);
 	}
 	else {
 		if(cnv->get_besitzer()==cnv->get_welt()->get_active_player()) {
@@ -93,16 +94,24 @@ convoi_detail_t::zeichnen(koord pos, koord gr)
 		int offset_y = pos.y+14+16;
 
 		// current value
-		char tmp[256];
+		char tmp[512];
 
 		// current power
 		sprintf( tmp, translator::translate("Leistung: %d kW"), cnv->get_sum_leistung() );
 		display_proportional_clip( pos.x+10, offset_y, tmp, ALIGN_LEFT, MONEY_PLUS, true );
 		offset_y += LINESPACE;
 
-		char buf[32];
-		money_to_string( buf, cnv->calc_restwert()/100.0 );
-		sprintf( tmp, "%s %s", translator::translate("Restwert:"), buf );
+		char number[64];
+		number_to_string( number, cnv->get_total_distance_traveled(), 0 );
+		sprintf( tmp, translator::translate("Odometer: %s km"), number );
+		display_proportional_clip( pos.x+10, offset_y, tmp, ALIGN_LEFT, MONEY_PLUS, true );
+		offset_y += LINESPACE;
+
+		sprintf( tmp, "%s %i", translator::translate("Station tiles:"), cnv->get_tile_length() );
+		display_proportional_clip( pos.x+10, offset_y, tmp, ALIGN_LEFT, MONEY_PLUS, true );
+		offset_y += LINESPACE;
+
+		money_to_string( tmp+sprintf( tmp, "%s ", translator::translate("Restwert:") ), cnv->calc_restwert()/100.0 );
 		display_proportional_clip( pos.x+10, offset_y, tmp, ALIGN_LEFT, MONEY_PLUS, true );
 		offset_y += LINESPACE;
 	}
@@ -114,17 +123,15 @@ convoi_detail_t::zeichnen(koord pos, koord gr)
  * This method is called if an action is triggered
  * @author Markus Weber
  */
-bool
-convoi_detail_t::action_triggered(gui_action_creator_t *komp,value_t /* */)           // 28-Dec-01    Markus Weber    Added
+bool convoi_detail_t::action_triggered(gui_action_creator_t *komp,value_t /* */)           // 28-Dec-01    Markus Weber    Added
 {
 	if(cnv.is_bound()) {
 		if(komp==&sale_button) {
-			cnv->self_destruct();
+			cnv->call_convoi_tool( 'x', NULL );
 			return true;
 		}
 		else if(komp==&withdraw_button) {
-			cnv->set_withdraw(!cnv->get_withdraw());
-			cnv->set_no_load(cnv->get_withdraw());
+			cnv->call_convoi_tool( 'w', NULL );
 			return true;
 		}
 	}
@@ -134,17 +141,87 @@ convoi_detail_t::action_triggered(gui_action_creator_t *komp,value_t /* */)
 
 
 /**
- * Resize the contents of the window
+ * Set window size and adjust component sizes and/or positions accordingly
  * @author Markus Weber
  */
-void convoi_detail_t::resize(const koord delta)
+void convoi_detail_t::set_fenstergroesse(koord groesse)
 {
-	gui_frame_t::resize(delta);
+	gui_frame_t::set_fenstergroesse(groesse);
 	scrolly.set_groesse(get_client_windowsize()-scrolly.get_pos());
 }
 
 
+// dummy for loading
+convoi_detail_t::convoi_detail_t(karte_t *w)
+: gui_frame_t("", NULL ),
+  scrolly(&veh_info),
+  veh_info(convoihandle_t())
+{
+	welt = w;
+}
 
+
+void convoi_detail_t::rdwr(loadsave_t *file)
+{
+	koord3d cnv_pos;
+	char name[128];
+	koord gr = get_fenstergroesse();
+	sint32 xoff = scrolly.get_scroll_x();
+	sint32 yoff = scrolly.get_scroll_y();
+	if(  file->is_saving()  ) {
+		cnv_pos = cnv->front()->get_pos();
+		tstrncpy(name, cnv->get_name(), lengthof(name));
+	}
+	cnv_pos.rdwr( file );
+	file->rdwr_str( name, lengthof(name) );
+	gr.rdwr( file );
+	file->rdwr_long( xoff );
+	file->rdwr_long( yoff );
+	if(  file->is_loading()  ) {
+		// find convoi by name and position
+		if(  grund_t *gr = welt->lookup(cnv_pos)  ) {
+			for(  uint8 i=0;  i<gr->get_top();  i++  ) {
+				if(  gr->obj_bei(i)->is_moving()  ) {
+					vehikel_t const* const v = ding_cast<vehikel_t>(gr->obj_bei(i));
+					if(  v  &&  v->get_convoi()  ) {
+						if(  strcmp(v->get_convoi()->get_name(),name)==0  ) {
+							cnv = v->get_convoi()->self;
+							break;
+						}
+					}
+				}
+			}
+		}
+		// we might be unlucky, then search all convois for a convoi with this name
+		if(  !cnv.is_bound()  ) {
+			for(  vector_tpl<convoihandle_t>::const_iterator i = welt->convois_begin(), end = welt->convois_end();  i != end;  ++i  ) {
+				if(  strcmp( (*i)->get_name(),name)==0  ) {
+					cnv = *i;
+					break;
+				}
+			}
+		}
+		// still not found?
+		if(  !cnv.is_bound()  ) {
+			dbg->error( "convoi_detail_t::rdwr()", "Could not restore convoi detail window of %s", name );
+			destroy_win( this );
+			return;
+		}
+		// now we can open the window ...
+		KOORD_VAL xpos = win_get_posx( this );
+		KOORD_VAL ypos = win_get_posy( this );
+		convoi_detail_t *w = new convoi_detail_t(cnv);
+		create_win( xpos, ypos, w, w_info, magic_convoi_detail+cnv.get_id() );
+		w->set_fenstergroesse( gr );
+		w->scrolly.set_scroll_position( xoff, yoff );
+		// we must invalidate halthandle
+		cnv = convoihandle_t();
+		destroy_win( this );
+	}
+}
+
+
+// component for vehicle display
 gui_vehicleinfo_t::gui_vehicleinfo_t(convoihandle_t cnv)
 {
 	this->cnv = cnv;
@@ -163,7 +240,7 @@ void gui_vehicleinfo_t::zeichnen(koord offset)
 		char buf[256], tmp[256];
 
 		// for bonus stuff
-		const sint32 ref_speed = cnv->get_welt()->get_average_speed( cnv->get_vehikel(0)->get_waytype() );
+		sint32 const ref_speed = cnv->get_welt()->get_average_speed(cnv->front()->get_waytype());
 		const sint32 speed_base = (100*speed_to_kmh(cnv->get_min_top_speed()))/ref_speed-100;
 
 		static cbuffer_t freight_info(1024);
@@ -249,3 +326,7 @@ void gui_vehicleinfo_t::zeichnen(koord offset)
 	// the size will change as soon something is loaded ...
 	set_groesse( get_groesse()+koord(0,total_height) );
 }
+
+
+
+
