@@ -18,23 +18,24 @@
 #include "../dataobj/translator.h"
 
 koord convoi_filter_frame_t::filter_buttons_pos[FILTER_BUTTONS] = {
-    koord(5,   4),
-    koord(5,  43),
-    koord(21, 58),
-    koord(21, 74),
-    koord(21, 90),
-    koord(21, 106),
-    koord(5,  175),
-    koord(21, 191),
-    koord(21, 207),
-    koord(21, 223),
-    koord(21, 239),
-    koord(21, 255),
-    koord(21, 271),
-    koord(125, 4),
-    koord(21, 122),
-    koord(21, 138),
-    koord(21, 154)
+	koord(5,   4),
+	koord(5,  43),
+	koord(21, 58),
+	koord(21, 74),
+	koord(21, 90),
+	koord(21, 106),
+	koord(5,  191),
+	koord(21, 207),
+	koord(21, 223),
+	koord(21, 239),
+	koord(21, 255),
+	koord(21, 271),
+	koord(21, 287),
+	koord(125, 4),
+	koord(21, 122),
+	koord(21, 138),
+	koord(21, 154),
+	koord(21, 170)
 };
 
 const char *convoi_filter_frame_t::filter_buttons_text[FILTER_BUTTONS] = {
@@ -54,7 +55,8 @@ const char *convoi_filter_frame_t::filter_buttons_text[FILTER_BUTTONS] = {
 	"clf_chk_waren",
 	"clf_chk_monorail",
 	"clf_chk_maglev",
-	"clf_chk_narrowgauge"
+	"clf_chk_narrowgauge",
+	"clf_chk_trams"
 };
 
 convoi_frame_t::filter_flag_t convoi_filter_frame_t::filter_buttons_types[FILTER_BUTTONS] = {
@@ -74,7 +76,8 @@ convoi_frame_t::filter_flag_t convoi_filter_frame_t::filter_buttons_types[FILTER
     convoi_frame_t::ware_filter,
     convoi_frame_t::monorail_filter,
     convoi_frame_t::maglev_filter,
-    convoi_frame_t::narrowgauge_filter
+    convoi_frame_t::narrowgauge_filter,
+	convoi_frame_t::tram_filter
 };
 
 
@@ -135,9 +138,9 @@ convoi_filter_frame_t::convoi_filter_frame_t(spieler_t *sp, convoi_frame_t *main
 		}
 	}
 	ware_cont.set_groesse(koord(100, 16*n + 4));
-	ware_scrolly.set_groesse(koord(125, 253));
+	ware_scrolly.set_groesse(koord(125, 269));
 
-	set_fenstergroesse(koord(255, 307));
+	set_fenstergroesse(koord(255, 323));
 }
 
 
