@@ -9,17 +9,18 @@
 
 #include "convoi_info_t.h"
 
-#include "../simconvoi.h"
 #include "../simdepot.h"
 #include "../vehicle/simvehikel.h"
 #include "../simcolor.h"
 #include "../simgraph.h"
 #include "../simworld.h"
+#include "../simmenu.h"
 #include "../simwin.h"
 
 #include "../dataobj/fahrplan.h"
 #include "../dataobj/translator.h"
 #include "../dataobj/umgebung.h"
+#include "../dataobj/loadsave.h"
 #include "fahrplan_gui.h"
 // @author hsiegeln
 #include "../simlinemgmt.h"
@@ -27,21 +28,32 @@
 #include "../boden/grund.h"
 #include "messagebox.h"
 
+#include "../player/simplay.h"
+
 #include "../utils/simstring.h"
 
 #include "components/list_button.h"
 
 #include "convoi_detail_t.h"
 
-const char cost_type[MAX_CONVOI_COST][64] =
+static const char cost_type[MAX_CONVOI_COST][64] =
 {
-	"Free Capacity",
-	"Transported",
-	"Revenue",
-	"Operation",
-	"Profit"
+	"Free Capacity", "Transported", "Revenue", "Operation", "Profit", "Distance"
 };
 
+static const int cost_type_color[MAX_CONVOI_COST] =
+{
+	COL_FREE_CAPACITY, COL_TRANSPORTED, COL_REVENUE, COL_OPERATION, COL_PROFIT, COL_DISTANCE
+};
+
+static const bool cost_type_money[MAX_CONVOI_COST] =
+{
+	false, false, true, true, true, false
+};
+
+
+karte_t *convoi_info_t::welt = NULL;
+
 bool convoi_info_t::route_search_in_progress=false;
 
 /**
@@ -59,10 +71,6 @@ const char *convoi_info_t::sort_text[SORT_MODES] = {
 	"Menge"
 };
 
-const int cost_type_color[MAX_CONVOI_COST] =
-{
-	COL_FREE_CAPACITY, COL_TRANSPORTED, COL_REVENUE, COL_OPERATION, COL_PROFIT
-};
 
 
 convoi_info_t::convoi_info_t(convoihandle_t cnv)
@@ -73,20 +81,83 @@ convoi_info_t::convoi_info_t(convoihandle_t cnv)
 			 " \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n"
 			 " \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n"
 			 " \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n \n"),
-	view(cnv->get_vehikel(0)),
+	view(cnv->front(), koord(max(64, get_base_tile_raster_width()), max(56, (get_base_tile_raster_width() * 7) / 8))),
 	sort_label(translator::translate("loaded passenger/freight")),
 	freight_info(8192)
 {
 	this->cnv = cnv;
+	welt = cnv->get_welt();
 	this->mean_convoi_speed = speed_to_kmh(cnv->get_akt_speed()*4);
 	this->max_convoi_speed = speed_to_kmh(cnv->get_min_top_speed()*4);
 
-	input.set_pos(koord(11,4));
-	input.set_text( cnv->access_internal_name(), 116);
+	const sint16 offset_below_viewport = 21 + view.get_groesse().y;
+	const sint16 total_width = 3*(BUTTON_WIDTH+BUTTON_SPACER) + 30 + view.get_groesse().x + 10;
+
+	input.set_pos(koord(10,4));
+	reset_cnv_name();
 	add_komponente(&input);
+	input.add_listener(this);
+
+	add_komponente(&view);
+
+	// this convoi doesn't belong to an AI
+	button.set_groesse(koord(BUTTON_WIDTH, BUTTON_HEIGHT));
+	button.set_text("Fahrplan");
+	button.set_typ(button_t::roundbox);
+	button.set_tooltip("Alters a schedule.");
+	add_komponente(&button);
+	button.set_pos(koord(BUTTON1_X,offset_below_viewport));
+	button.add_listener(this);
+
+	go_home_button.set_groesse(koord(BUTTON_WIDTH, BUTTON_HEIGHT));
+	go_home_button.set_pos(koord(BUTTON2_X,offset_below_viewport));
+	go_home_button.set_text("go home");
+	go_home_button.set_typ(button_t::roundbox);
+	go_home_button.set_tooltip("Sends the convoi to the last depot it departed from!");
+	add_komponente(&go_home_button);
+	go_home_button.add_listener(this);
+
+	no_load_button.set_groesse(koord(BUTTON_WIDTH, BUTTON_HEIGHT));
+	no_load_button.set_pos(koord(BUTTON3_X,offset_below_viewport));
+	no_load_button.set_text("no load");
+	no_load_button.set_typ(button_t::roundbox);
+	no_load_button.set_tooltip("No goods are loaded onto this convoi.");
+	add_komponente(&no_load_button);
+	no_load_button.add_listener(this);
+
+	follow_button.set_groesse(koord(view.get_groesse().x, BUTTON_HEIGHT));
+	follow_button.set_text("follow me");
+	follow_button.set_typ(button_t::roundbox_state);
+	follow_button.set_tooltip("Follow the convoi on the map.");
+	add_komponente(&follow_button);
+	follow_button.add_listener(this);
+
+	// chart
+	chart.set_pos(koord(88,offset_below_viewport+BUTTON_HEIGHT+8));
+	chart.set_groesse(koord(TOTAL_WIDTH-88-4, 100));
+	chart.set_dimension(12, 10000);
+	chart.set_visible(false);
+	chart.set_background(MN_GREY1);
+	for (int cost = 0; cost<MAX_CONVOI_COST; cost++) {
+		chart.add_curve( cost_type_color[cost], cnv->get_finance_history(), MAX_CONVOI_COST, cost, MAX_MONTHS, cost_type_money[cost], false, true, cost_type_money[cost]*2 );
+		filterButtons[cost].init(button_t::box_state, cost_type[cost], koord(BUTTON1_X+(BUTTON_WIDTH+BUTTON_SPACER)*(cost%4), view.get_groesse().y+174+(BUTTON_HEIGHT+2)*(cost/4)), koord(BUTTON_WIDTH, BUTTON_HEIGHT));
+		filterButtons[cost].add_listener(this);
+		filterButtons[cost].background = cost_type_color[cost];
+		filterButtons[cost].set_visible(false);
+		filterButtons[cost].pressed = false;
+		add_komponente(filterButtons + cost);
+	}
+	add_komponente(&chart);
 
 	add_komponente(&sort_label);
 
+	sort_button.set_groesse(koord(BUTTON_WIDTH, BUTTON_HEIGHT));
+	sort_button.set_text(sort_text[umgebung_t::default_sortmode]);
+	sort_button.set_typ(button_t::roundbox);
+	sort_button.add_listener(this);
+	sort_button.set_tooltip("Sort by");
+	add_komponente(&sort_button);
+
 	toggler.set_groesse(koord(BUTTON_WIDTH, BUTTON_HEIGHT));
 	toggler.set_text("Chart");
 	toggler.set_typ(button_t::roundbox_state);
@@ -95,13 +166,6 @@ convoi_info_t::convoi_info_t(convoihandle_t cnv)
 	add_komponente(&toggler);
 	toggler.pressed = false;
 
-	sort_button.set_groesse(koord(BUTTON_WIDTH, BUTTON_HEIGHT));
-	sort_button.set_text(sort_text[umgebung_t::default_sortmode]);
-	sort_button.set_typ(button_t::roundbox);
-	sort_button.add_listener(this);
-	sort_button.set_tooltip("Sort by");
-	add_komponente(&sort_button);
-
 	details_button.set_groesse(koord(BUTTON_WIDTH, BUTTON_HEIGHT));
 	details_button.set_text("Details");
 	details_button.set_typ(button_t::roundbox);
@@ -109,7 +173,7 @@ convoi_info_t::convoi_info_t(convoihandle_t cnv)
 	details_button.set_tooltip("Vehicle details");
 	add_komponente(&details_button);
 
-	scrolly.set_pos(koord(0, 122));
+	scrolly.set_pos(koord(0, offset_below_viewport+46));
 	add_komponente(&scrolly);
 
 	filled_bar.add_color_value(&cnv->get_loading_limit(), COL_YELLOW);
@@ -131,66 +195,22 @@ convoi_info_t::convoi_info_t(convoihandle_t cnv)
 	line_button.add_listener( this );
 	line_bound = false;
 
-	set_fenstergroesse(koord(TOTAL_WIDTH, 278));
-
-	// chart
-	chart.set_pos(koord(44,76+BUTTON_HEIGHT+8));
-	chart.set_groesse(koord(TOTAL_WIDTH-44-4, 100));
-	chart.set_dimension(12, 10000);
-	chart.set_visible(false);
-	chart.set_background(MN_GREY1);
-	for (int cost = 0; cost<MAX_CONVOI_COST; cost++) {
-		chart.add_curve(cost_type_color[cost], cnv->get_finance_history(), MAX_CONVOI_COST, cost, MAX_MONTHS, cost<MAX_CONVOI_NON_MONEY_TYPES ? 0 : 1, false, true);
-		filterButtons[cost].init(button_t::box_state, cost_type[cost], koord(BUTTON1_X+(BUTTON_WIDTH+BUTTON_SPACER)*(cost%4), 230+(BUTTON_HEIGHT+2)*(cost/4)), koord(BUTTON_WIDTH, BUTTON_HEIGHT));
-		filterButtons[cost].add_listener(this);
-		filterButtons[cost].background = cost_type_color[cost];
-		filterButtons[cost].set_visible(false);
-		filterButtons[cost].pressed = false;
-		add_komponente(filterButtons + cost);
-	}
-	add_komponente(&chart);
-	add_komponente(&view);
-
-	// this convoi belongs not to an AI
-	button.set_groesse(koord(BUTTON_WIDTH, BUTTON_HEIGHT));
-	button.set_text("Fahrplan");
-	button.set_typ(button_t::roundbox);
-	button.set_tooltip("Alters a schedule.");
-	add_komponente(&button);
-	button.set_pos(koord(BUTTON1_X,76));
-	button.add_listener(this);
-
-	go_home_button.set_groesse(koord(BUTTON_WIDTH, BUTTON_HEIGHT));
-	go_home_button.set_pos(koord(BUTTON2_X,76));
-	go_home_button.set_text("go home");
-	go_home_button.set_typ(button_t::roundbox);
-	go_home_button.set_tooltip("Sends the convoi to the last depot it departed from!");
-	add_komponente(&go_home_button);
-	go_home_button.add_listener(this);
-
-	no_load_button.set_groesse(koord(BUTTON_WIDTH, BUTTON_HEIGHT));
-	no_load_button.set_pos(koord(BUTTON3_X,76));
-	no_load_button.set_text("no load");
-	no_load_button.set_typ(button_t::roundbox);
-	no_load_button.set_tooltip("No goods are loaded onto this convoi.");
-	add_komponente(&no_load_button);
-	no_load_button.add_listener(this);
-
-	follow_button.set_groesse(koord(66, BUTTON_HEIGHT));
-	follow_button.set_text("follow me");
-	follow_button.set_typ(button_t::roundbox_state);
-	follow_button.set_tooltip("Follow the convoi on the map.");
-	add_komponente(&follow_button);
-	follow_button.add_listener(this);
+	set_fenstergroesse(koord(total_width, view.get_groesse().y+222));
 
 	cnv->set_sortby( umgebung_t::default_sortmode );
 
-	set_min_windowsize(koord(TOTAL_WIDTH, 194));
+	set_min_windowsize(koord(total_width, view.get_groesse().y+138));
 	set_resizemode(diagonal_resize);
 	resize(koord(0,0));
 }
 
 
+// only handle a pending renaming ...
+convoi_info_t::~convoi_info_t()
+{
+	// rename if necessary
+	rename_cnv();
+}
 
 
 /**
@@ -199,11 +219,10 @@ convoi_info_t::convoi_info_t(convoihandle_t cnv)
  * in dem die Komponente dargestellt wird.
  * @author Hj. Malthaner
  */
-void
-convoi_info_t::zeichnen(koord pos, koord gr)
+void convoi_info_t::zeichnen(koord pos, koord gr)
 {
 	if(!cnv.is_bound()) {
-		destroy_win(dynamic_cast <gui_fenster_t *> (this));
+		destroy_win(this);
 	}
 	else {
 		if(cnv->get_besitzer()==cnv->get_welt()->get_active_player()) {
@@ -217,11 +236,13 @@ convoi_info_t::zeichnen(koord pos, koord gr)
 			}
 			button.enable();
 			go_home_button.pressed = route_search_in_progress;
-			if(  cnv->get_schedule()->get_count() > 0  ) {
+			details_button.pressed = win_get_magic( magic_convoi_detail+cnv.get_id() );
+			if (!cnv->get_schedule()->empty()) {
 				const grund_t* g = cnv->get_welt()->lookup(cnv->get_schedule()->get_current_eintrag().pos);
 				if (g != NULL && g->get_depot()) {
 					go_home_button.disable();
-				} else {
+				}
+				else {
 					goto enable_home;
 				}
 			}
@@ -248,8 +269,8 @@ enable_home:
 		cnv->get_freight_info(freight_info);
 		text.set_text(freight_info);
 
-		route_bar.set_base(cnv->get_route()->get_max_n());
-		cnv_route_index = cnv->get_vehikel(0)->get_route_index()-1;
+		route_bar.set_base(cnv->get_route()->get_count()-1);
+		cnv_route_index = cnv->front()->get_route_index() - 1;
 
 		// all gui stuff set => display it
 		gui_frame_t::zeichnen(pos, gr);
@@ -264,34 +285,36 @@ enable_home:
 		mean_convoi_speed += speed_to_kmh(cnv->get_akt_speed()*4);
 		mean_convoi_speed /= 2;
 		sprintf(tmp,translator::translate("%i km/h (max. %ikm/h)"), (mean_convoi_speed+3)/4, speed_to_kmh(cnv->get_min_top_speed()) );
-		display_proportional( pos.x+11, pos.y+16+20, tmp, ALIGN_LEFT, COL_BLACK, true );
+		display_proportional( pos.x+10, pos.y+16+20, tmp, ALIGN_LEFT, COL_BLACK, true );
 
 		// next important: income stuff
-		int len = display_proportional(pos.x + 11, pos.y + 16 + 20 + 1 * LINESPACE, translator::translate("Gewinn"), ALIGN_LEFT, COL_BLACK, true ) + 5;
+		int len = display_proportional(pos.x + 10, pos.y + 16 + 20 + 1 * LINESPACE, translator::translate("Gewinn"), ALIGN_LEFT, COL_BLACK, true ) + 5;
 		money_to_string( tmp, cnv->get_jahresgewinn()/100.0 );
-		len += display_proportional( pos.x+11+len, pos.y+16+20+1*LINESPACE, tmp, ALIGN_LEFT, cnv->get_jahresgewinn()>0?MONEY_PLUS:MONEY_MINUS, true )+5;
-		sprintf(tmp," (%1.2f$/km)", cnv->get_running_cost()/100.0 );
-		display_proportional( pos.x+11+len, pos.y+16+20+1*LINESPACE, tmp, ALIGN_LEFT, COL_BLACK, true );
+		len += display_proportional( pos.x+10+len, pos.y+16+20+1*LINESPACE, tmp, ALIGN_LEFT, cnv->get_jahresgewinn()>0?MONEY_PLUS:MONEY_MINUS, true )+5;
+		money_to_string( tmp+1, cnv->get_running_cost()/100.0 );
+		strcat( tmp, "/km)" );
+		tmp[0] = '(';
+		display_proportional( pos.x+10+len, pos.y+16+20+1*LINESPACE, tmp, ALIGN_LEFT, COL_BLACK, true );
 
 		// the weight entry
 		info_buf.clear();
 		info_buf.append( translator::translate("Gewicht") );
 		info_buf.append( ": " );
-		info_buf.append( cnv->get_sum_gesamtgewicht() );
-		info_buf.append( " (" );
-		info_buf.append( cnv->get_sum_gesamtgewicht()-cnv->get_sum_gewicht() );
-		info_buf.append( ") t" );
-		display_proportional( pos.x+11, pos.y+16+20+2*LINESPACE, info_buf, ALIGN_LEFT, COL_BLACK, true );
+		info_buf.append( cnv->get_sum_gesamtgewicht(), 0 );
+		info_buf.append( "t (" );
+		info_buf.append( cnv->get_sum_gesamtgewicht()-cnv->get_sum_gewicht(), 0 );
+		info_buf.append( "t)" );
+		display_proportional( pos.x+10, pos.y+16+20+2*LINESPACE, info_buf, ALIGN_LEFT, COL_BLACK, true );
 
 		// next stop
 		const schedule_t * fpl = cnv->get_schedule();
 		info_buf.clear();
 		info_buf.append(translator::translate("Fahrtziel"));
-		fahrplan_gui_t::gimme_short_stop_name(info_buf, cnv->get_welt(), fpl, fpl->get_aktuell(), 34);
-		len = display_proportional_clip( pos.x+11, pos.y+16+20+3*LINESPACE, info_buf, ALIGN_LEFT, COL_BLACK, true );
+		fahrplan_gui_t::gimme_short_stop_name(info_buf, cnv->get_welt(), cnv->get_besitzer(), fpl, fpl->get_aktuell(), 34);
+		len = display_proportional_clip( pos.x+10, pos.y+16+20+3*LINESPACE, info_buf, ALIGN_LEFT, COL_BLACK, true );
 
 		// convoi load indicator
-		const int offset = max( len, 167)+3;
+		const int offset = max( len+10, 167)+3;
 		route_bar.set_pos(koord(offset,22+3*LINESPACE));
 		route_bar.set_groesse(koord(view.get_pos().x-offset-5, 4));
 
@@ -309,6 +332,22 @@ enable_home:
 }
 
 
+// activate the statistic
+void convoi_info_t::show_hide_statistics( bool show )
+{
+	toggler.pressed = show;
+	const koord offset = show ? koord(0, 170) : koord(0, -170);
+	set_min_windowsize(get_min_windowsize() + offset);
+	scrolly.set_pos(scrolly.get_pos() + offset);
+	chart.set_visible(show);
+	set_fenstergroesse(get_fenstergroesse() + offset);
+	resize(koord(0,0));
+	for (int i=0;i<MAX_CONVOI_COST;i++) {
+		filterButtons[i].set_visible(toggler.pressed);
+	}
+}
+
+
 /**
  * This method is called if an action is triggered
  * @author Hj. Malthaner
@@ -327,9 +366,9 @@ bool convoi_info_t::action_triggered( gui_action_creator_t *komp,value_t /* */)
 		return true;
 	}
 
-	// datails?
+	// details?
 	if(komp == &details_button) {
-		create_win(20, 20, new convoi_detail_t(cnv), w_info, (long)this);
+		create_win(20, 20, new convoi_detail_t(cnv), w_info, magic_convoi_detail+cnv.get_id() );
 		return true;
 	}
 
@@ -338,6 +377,11 @@ bool convoi_info_t::action_triggered( gui_action_creator_t *komp,value_t /* */)
 		cnv->get_welt()->set_dirty();
 	}
 
+	if(  komp == &input  ) {
+		// rename if necessary
+		rename_cnv();
+	}
+
 	// sort by what
 	if(komp == &sort_button) {
 		// sort by what
@@ -350,19 +394,16 @@ bool convoi_info_t::action_triggered( gui_action_creator_t *komp,value_t /* */)
 	if(cnv->get_besitzer()==cnv->get_welt()->get_active_player()) {
 
 		if(komp == &button) {
-			cnv->open_schedule_window();
+			cnv->call_convoi_tool( 'f', NULL );
 			return true;
 		}
 
 		if(komp == &no_load_button    &&    !route_search_in_progress) {
-			cnv->set_no_load(!cnv->get_no_load());
-			if(!cnv->get_no_load()) {
-				cnv->set_withdraw(false);
-			}
+			cnv->call_convoi_tool( 'n', NULL );
 			return true;
 		}
 
-		if(komp == &go_home_button    &&    !route_search_in_progress) {
+		if(komp == &go_home_button  &&  !route_search_in_progress) {
 			// limit update to certain states that are considered to be save for fahrplan updates
 			int state = cnv->get_state();
 			if(state==convoi_t::FAHRPLANEINGABE) {
@@ -378,59 +419,48 @@ DBG_MESSAGE("convoi_info_t::action_triggered()","convoi state %i => cannot chang
 			koord3d home = koord3d(0,0,0);
 			while (depot_iter.next()) {
 				depot_t *depot = depot_iter.get_current();
-				if(depot->get_wegtyp()!=cnv->get_vehikel(0)->get_besch()->get_waytype()    ||    depot->get_besitzer()!=cnv->get_besitzer()) {
+				vehikel_t& v = *cnv->front();
+				if (depot->get_wegtyp()   != v.get_besch()->get_waytype() ||
+						depot->get_besitzer() != cnv->get_besitzer()) {
 					continue;
 				}
 				koord3d pos = depot->get_pos();
-				if(!shortest_route->empty()    &&    abs_distance(pos.get_2d(),cnv->get_pos().get_2d())>=shortest_route->get_max_n()) {
+				if(!shortest_route->empty()    &&    koord_distance(pos.get_2d(),cnv->get_pos().get_2d())>=shortest_route->get_count()-1) {
 					// the current route is already shorter, no need to search further
 					continue;
 				}
-				bool found = cnv->get_vehikel(0)->calc_route(cnv->get_pos(), pos,    50, route); // do not care about speed
-				if (found) {
-					if(  route->get_max_n() < shortest_route->get_max_n()    ||    shortest_route->empty()  ) {
+				if (v.calc_route(cnv->get_pos(), pos, 50, route)) { // do not care about speed
+					if(  route->get_count() < shortest_route->get_count()    ||    shortest_route->empty()  ) {
 						shortest_route->kopiere(route);
 						home = pos;
 					}
 				}
 			}
 			delete route;
-			DBG_MESSAGE("shortest route has ", "%i hops", shortest_route->get_max_n());
+			DBG_MESSAGE("shortest route has ", "%i hops", shortest_route->get_count()-1);
 
 			// if route to a depot has been found, update the convoi's schedule
-			bool b_depot_found = false;
+			const char *txt;
 			if(!shortest_route->empty()) {
-				schedule_t *fpl = cnv->get_schedule();
+				schedule_t *fpl = cnv->get_schedule()->copy();
 				fpl->insert(cnv->get_welt()->lookup(home));
 				fpl->set_aktuell( (fpl->get_aktuell()+fpl->get_count()-1)%fpl->get_count() );
-				b_depot_found = cnv->set_schedule(fpl);
+				cbuffer_t buf(5500);
+				fpl->sprintf_schedule( buf );
+				cnv->call_convoi_tool( 'g', buf );
+				txt = "Convoi has been sent\nto the nearest depot\nof appropriate type.\n";
+			}
+			else {
+				txt = "Home depot not found!\nYou need to send the\nconvoi to the depot\nmanually.";
 			}
 			delete shortest_route;
 			route_search_in_progress = false;
-
-			// show result
-			const char* txt;
-			if (b_depot_found) {
-				txt = "Convoi has been sent\nto the nearest depot\nof appropriate type.\n";
-			} else {
-				txt = "Home depot not found!\nYou need to send the\nconvoi to the depot\nmanually.";
-			}
 			create_win( new news_img(txt), w_time_delete, magic_none);
 		} // end go home button
 	}
 
 	if (komp == &toggler) {
-		toggler.pressed = !toggler.pressed;
-		const koord offset = toggler.pressed ? koord(0, 170) : koord(0, -170);
-		set_min_windowsize( koord(TOTAL_WIDTH, toggler.pressed ? 364: 194));
-		scrolly.set_pos( scrolly.get_pos()+koord(0,offset.y) );
-		// toggle visibility of components
-		chart.set_visible(toggler.pressed);
-		set_fenstergroesse(get_fenstergroesse() + offset);
-		resize(koord(0,0));
-		for (int i=0;i<MAX_CONVOI_COST;i++) {
-			filterButtons[i].set_visible(toggler.pressed);
-		}
+		show_hide_statistics( toggler.pressed^1 );
 		return true;
 	}
 
@@ -439,7 +469,8 @@ DBG_MESSAGE("convoi_info_t::action_triggered()","convoi state %i => cannot chang
 			filterButtons[i].pressed = !filterButtons[i].pressed;
 			if(filterButtons[i].pressed) {
 				chart.show_curve(i);
-			} else {
+			}
+			else {
 				chart.hide_curve(i);
 			}
 
@@ -451,22 +482,55 @@ DBG_MESSAGE("convoi_info_t::action_triggered()","convoi state %i => cannot chang
 }
 
 
+void convoi_info_t::reset_cnv_name()
+{
+	// change text input of selected line
+	if (cnv.is_bound()) {
+		tstrncpy(old_cnv_name, cnv->get_name(), sizeof(old_cnv_name));
+		tstrncpy(cnv_name, cnv->get_name(), sizeof(cnv_name));
+		input.set_text(cnv_name, sizeof(cnv_name));
+	}
+}
+
+
+void convoi_info_t::rename_cnv()
+{
+	if (cnv.is_bound()) {
+		const char *t = input.get_text();
+		// only change if old name and current name are the same
+		// otherwise some unintended undo if renaming would occur
+		if(  t  &&  t[0]  &&  strcmp(t, cnv->get_name())  &&  strcmp(old_cnv_name, cnv->get_name())==0) {
+			// text changed => call tool
+			cbuffer_t buf(300);
+			buf.printf( "c%u,%s", cnv.get_id(), t );
+			werkzeug_t *w = create_tool( WKZ_RENAME_TOOL | SIMPLE_TOOL );
+			w->set_default_param( buf );
+			cnv->get_welt()->set_werkzeug( w, cnv->get_besitzer());
+			// since init always returns false, it is save to delete immediately
+			delete w;
+			// do not trigger this command again
+			tstrncpy(old_cnv_name, t, sizeof(old_cnv_name));
+		}
+	}
+}
+
+
 /**
- * Resize the contents of the window
+ * Set window size and adjust component sizes and/or positions accordingly
  * @author Markus Weber
  */
-void convoi_info_t::resize(const koord delta)
+void convoi_info_t::set_fenstergroesse(koord groesse)
 {
-	gui_frame_t::resize(delta);
+	gui_frame_t::set_fenstergroesse(groesse);
 
-	input.set_groesse(koord(get_fenstergroesse().x-22, 13));
+	input.set_groesse(koord(get_fenstergroesse().x - 20, 13));
 
-	view.set_pos(koord(get_fenstergroesse().x - 64 - 12 , 21));
-	follow_button.set_pos(koord(view.get_pos().x-1,77));
+	view.set_pos(koord(get_fenstergroesse().x - view.get_groesse().x - 10 , 21));
+	follow_button.set_pos(koord(view.get_pos().x, view.get_groesse().y + 21));
 
 	scrolly.set_groesse(get_client_windowsize()-scrolly.get_pos());
 
-	const int yoff = scrolly.get_pos().y-BUTTON_HEIGHT-2;
+	const sint16 yoff = scrolly.get_pos().y-BUTTON_HEIGHT-2;
 	sort_button.set_pos(koord(BUTTON1_X,yoff));
 	toggler.set_pos(koord(BUTTON3_X,yoff));
 	details_button.set_pos(koord(BUTTON4_X,yoff));
@@ -480,3 +544,102 @@ void convoi_info_t::resize(const koord delta)
 	filled_bar.set_pos(koord(170,22+2*LINESPACE));
 	filled_bar.set_groesse(koord(view.get_pos().x - 175, 4));
 }
+
+
+
+convoi_info_t::convoi_info_t(karte_t *welt)
+:	gui_frame_t("", NULL),
+	scrolly(&text),
+	text(""),
+	view( welt, koord(64,64)),
+	sort_label(translator::translate("loaded passenger/freight")),
+	freight_info(0)
+{
+	this->welt = welt;
+}
+
+
+
+void convoi_info_t::rdwr(loadsave_t *file)
+{
+	koord3d cnv_pos;
+	char name[128];
+	koord gr = get_fenstergroesse();
+	uint32 flags = 0;
+	bool stats = toggler.pressed;
+	sint32 xoff = scrolly.get_scroll_x();
+	sint32 yoff = scrolly.get_scroll_y();
+	if(  file->is_saving()  ) {
+		cnv_pos = cnv->front()->get_pos();
+		for( int i = 0; i<MAX_CONVOI_COST; i++) {
+			if(  filterButtons[i].pressed  ) {
+				flags |= (1<<i);
+			}
+		}
+		tstrncpy(name, cnv->get_name(), lengthof(name));
+	}
+	cnv_pos.rdwr( file );
+	file->rdwr_str( name, lengthof(name) );
+	gr.rdwr( file );
+	file->rdwr_long( flags );
+	file->rdwr_byte( umgebung_t::default_sortmode );
+	file->rdwr_bool( stats );
+	file->rdwr_long( xoff );
+	file->rdwr_long( yoff );
+	if(  file->is_loading()  ) {
+		// find convoi by name and position
+		if(  grund_t *gr = welt->lookup(cnv_pos)  ) {
+			for(  uint8 i=0;  i<gr->get_top();  i++  ) {
+				if(  gr->obj_bei(i)->is_moving()  ) {
+					vehikel_t const* const v = ding_cast<vehikel_t>(gr->obj_bei(i));
+					if(  v  &&  v->get_convoi()  ) {
+						if(  strcmp(v->get_convoi()->get_name(),name)==0  ) {
+							cnv = v->get_convoi()->self;
+							break;
+						}
+					}
+				}
+			}
+		}
+		// we might be unlucky, then search all convois for a convoi with this name
+		if(  !cnv.is_bound()  ) {
+			for(  vector_tpl<convoihandle_t>::const_iterator i = welt->convois_begin(), end = welt->convois_end();  i != end;  ++i  ) {
+				if(  strcmp( (*i)->get_name(),name)==0  ) {
+					cnv = *i;
+					break;
+				}
+			}
+		}
+		// still not found?
+		if(  !cnv.is_bound()  ) {
+			dbg->error( "convoi_info_t::rdwr()", "Could not restore convoi info window of %s", name );
+			destroy_win( this );
+			return;
+		}
+		// now we can open the window ...
+		KOORD_VAL xpos = win_get_posx( this );
+		KOORD_VAL ypos = win_get_posy( this );
+		convoi_info_t *w = new convoi_info_t(cnv);
+		create_win( xpos, ypos, w, w_info, magic_convoi_info+cnv.get_id() );
+		if(  stats  ) {
+			gr.y -= 170;
+		}
+		w->set_fenstergroesse( gr );
+		for( int i = 0; i<MAX_CONVOI_COST; i++) {
+			w->filterButtons[i].pressed = (flags>>i)&1;
+			if(w->filterButtons[i].pressed) {
+				w->chart.show_curve(i);
+			}
+		}
+		if(  stats  ) {
+			w->show_hide_statistics( true );
+		}
+		cnv->get_freight_info(w->freight_info);
+		w->text.set_text(w->freight_info);
+		w->text.recalc_size();
+		w->scrolly.set_scroll_position( xoff, yoff );
+		// we must invalidate halthandle
+		cnv = convoihandle_t();
+		destroy_win( this );
+	}
+}
