@@ -1,5 +1,5 @@
 /*
- * dialog zur Eingabe eines Fahrplanes
+ * Dialog window for defining a schedule
  *
  * Hj. Malthaner
  *
@@ -14,8 +14,9 @@
 #include "../simhalt.h"
 #include "../simskin.h"
 #include "../simworld.h"
-#include "../simwerkz.h"
+#include "../simmenu.h"
 #include "../simgraph.h"
+#include "../simtools.h"
 
 #include "../utils/simstring.h"
 #include "../utils/cbuffer_t.h"
@@ -23,8 +24,14 @@
 #include "../boden/grund.h"
 
 #include "../dataobj/fahrplan.h"
+#include "../dataobj/loadsave.h"
 #include "../dataobj/translator.h"
 
+#include "../player/simplay.h"
+#include "../vehicle/simvehikel.h"
+
+#include "../tpl/vector_tpl.h"
+
 #include "fahrplan_gui.h"
 #include "line_item.h"
 #include "components/list_button.h"
@@ -32,76 +39,69 @@
 #include "karte.h"
 
 char fahrplan_gui_t::no_line[128];	// contains the current translation of "<no line>"
+karte_t *fahrplan_gui_t::welt = NULL;
 
 
 /**
- * Fuellt buf mit Beschreibung des i-ten Eintrages des Fahrplanes
+ * Fills buf with description of schedule's i'th entry.
  *
  * @author Hj. Malthaner
  */
-void fahrplan_gui_t::gimme_stop_name(cbuffer_t & buf, karte_t *welt, const linieneintrag_t &entry )
+void fahrplan_gui_t::gimme_stop_name(cbuffer_t & buf, karte_t *welt, const spieler_t *sp, const linieneintrag_t &entry )
 {
-	const grund_t* gr = welt->lookup(entry.pos);
-	if(gr==NULL) {
-		buf.printf("%s (%s)", translator::translate("Invalid coordinate"), entry.pos.get_str() );
-	}
-	else {
-		halthandle_t halt = gr->ist_wasser() ? haltestelle_t::get_halt(welt, entry.pos) : gr->get_halt();
-
-		if(halt.is_bound()) {
-			if (entry.ladegrad != 0) {
-				buf.printf("%d%% %s (%s)",
-					entry.ladegrad,
-					halt->get_name(),
-					entry.pos.get_str() );
-			}
-			else {
-				buf.printf("%s (%s)",
-					halt->get_name(),
-					entry.pos.get_str() );
-			}
+	halthandle_t halt = haltestelle_t::get_halt(welt, entry.pos, sp);
+	if(halt.is_bound()) {
+		if (entry.ladegrad != 0) {
+			buf.printf("%d%% %s (%s)", entry.ladegrad, halt->get_name(), entry.pos.get_str() );
 		}
 		else {
-			if(gr->get_depot() != NULL) {
-				buf.printf("%s (%s)", translator::translate("Depot"), entry.pos.get_str() );
-			}
-			else {
-				buf.printf("%s (%s)", translator::translate("Wegpunkt"), entry.pos.get_str() );
-			}
+			buf.printf("%s (%s)",
+				halt->get_name(),
+				entry.pos.get_str() );
+		}
+	}
+	else {
+		const grund_t* gr = welt->lookup(entry.pos);
+		if(gr==NULL) {
+			buf.printf("%s (%s)", translator::translate("Invalid coordinate"), entry.pos.get_str() );
+		}
+		else if(gr->get_depot() != NULL) {
+			buf.printf("%s (%s)", translator::translate("Depot"), entry.pos.get_str() );
+		}
+		else {
+			buf.printf("%s (%s)", translator::translate("Wegpunkt"), entry.pos.get_str() );
 		}
 	}
 }
 
 
 /**
- * Fuellt buf mit Beschreibung des i-ten Eintrages des Fahrplanes
+ * Fills buf with description of schedule's i'th entry.
  * short version, without loading level and position ...
  * @author Hj. Malthaner
  */
-void fahrplan_gui_t::gimme_short_stop_name(cbuffer_t &buf, karte_t *welt, const schedule_t *fpl, int i, int max_chars)
+void fahrplan_gui_t::gimme_short_stop_name(cbuffer_t &buf, karte_t *welt, const spieler_t *sp, const schedule_t *fpl, int i, int max_chars)
 {
 	if(i<0  ||  fpl==NULL  ||  i>=fpl->get_count()) {
-		dbg->warning("void fahrplan_gui_t::gimme_stop_name()","tried to recieved unused entry %i in schedule %p.",i,fpl);
+		dbg->warning("void fahrplan_gui_t::gimme_short_stop_name()","tried to receive unused entry %i in schedule %p.",i,fpl);
 		return;
 	}
 	const linieneintrag_t& entry = fpl->eintrag[i];
-	const grund_t* gr = welt->lookup(entry.pos);
 	const char *p;
-	if(gr==NULL) {
-		p = translator::translate("Invalid coordinate");
+	halthandle_t halt = haltestelle_t::get_halt(welt, entry.pos, sp);
+	if(halt.is_bound()) {
+		p = halt->get_name();
 	}
 	else {
-		halthandle_t halt = haltestelle_t::get_halt(welt, entry.pos);
-		if(halt.is_bound()) {
-			p = halt->get_name();
+		const grund_t* gr = welt->lookup(entry.pos);
+		if(gr==NULL) {
+			p = translator::translate("Invalid coordinate");
+		}
+		else if(gr->get_depot() != NULL) {
+			p = translator::translate("Depot");
 		}
 		else {
-			if(gr->get_depot() != NULL) {
-				p = translator::translate("Depot");
-			}
-			else {
-				p = translator::translate("Wegpunkt");
-			}
+			p = translator::translate("Wegpunkt");
 		}
 	}
 	// finally append
@@ -125,27 +125,20 @@ void fahrplan_gui_stats_t::zeichnen(koord offset)
 {
 	if(fpl) {
 		sint16 width = 16;
-		image_id const arrow_right_normal = skinverwaltung_t::window_skin->get_bild(10)->get_nummer();
 
 		for (int i = 0; i < fpl->get_count(); i++) {
 
 			buf.clear();
 			buf.printf( "%i) ", i+1 );
-			fahrplan_gui_t::gimme_stop_name( buf, welt, fpl->eintrag[i] );
+			fahrplan_gui_t::gimme_stop_name( buf, welt, sp, fpl->eintrag[i] );
 			sint16 w = display_proportional_clip(offset.x + 4 + 10, offset.y + i * (LINESPACE + 1) + 2, buf, ALIGN_LEFT, COL_BLACK, true);
 			if(  w>width  ) {
 				width = w;
 			}
 
-			if(i!=fpl->get_aktuell()) {
-				// goto information
-				display_color_img(arrow_right_normal, offset.x + 2, offset.y + i * (LINESPACE + 1), 0, false, true);
-			}
-			else {
-				// select goto button
-				display_color_img(skinverwaltung_t::window_skin->get_bild(11)->get_nummer(),
-					offset.x + 2, offset.y + i * (LINESPACE + 1), 0, false, true);
-			}
+			// the goto button (right arrow)
+			display_color_img( i!=fpl->get_aktuell() ? button_t::arrow_right_normal : button_t::arrow_right_pushed,
+				offset.x + 2, offset.y + i * (LINESPACE + 1), 0, false, true);
 		}
 		set_groesse( koord(width+11, fpl->get_count() * (LINESPACE + 1) ) );
 	}
@@ -155,9 +148,13 @@ void fahrplan_gui_stats_t::zeichnen(koord offset)
 
 fahrplan_gui_t::~fahrplan_gui_t()
 {
-	update_werkzeug( false );
-	// hide schedule on minimap (may not current, but for safe)
-	reliefkarte_t::get_karte()->set_current_fpl(NULL, 0); // (*fpl,player_nr)
+	if(  sp  ) {
+		update_werkzeug( false );
+		// hide schedule on minimap (may not current, but for safe)
+		reliefkarte_t::get_karte()->set_current_fpl(NULL, 0); // (*fpl,player_nr)
+	}
+	delete fpl;
+
 }
 
 
@@ -168,22 +165,24 @@ fahrplan_gui_t::fahrplan_gui_t(schedule_t* fpl_, spieler_t* sp_, convoihandle_t
 	lb_wait("month wait time"),
 	lb_waitlevel(NULL, COL_WHITE, gui_label_t::right),
 	lb_load("Full load"),
-	stats(sp_->get_welt()),
+	stats(sp_->get_welt(),sp_),
 	scrolly(&stats),
-	fpl(fpl_),
+	old_fpl(fpl_),
 	sp(sp_),
 	cnv(cnv_)
 {
+	welt = sp->get_welt();
+	old_fpl->eingabe_beginnen();
+	fpl = old_fpl->copy();
 	stats.set_fahrplan(fpl);
 	if(!cnv.is_bound()) {
-		new_line = linehandle_t();
+		old_line = new_line = linehandle_t();
 		show_line_selector(false);
 	}
 	else {
-		new_line = cnv_->get_line();
+		old_line = new_line = cnv_->get_line();
 	}
-	this->fpl = fpl;
-	fpl->eingabe_beginnen();
+	old_line_count = 0;
 	strcpy(no_line, translator::translate("<no line>"));
 
 	sint16 ypos = 0;
@@ -288,52 +287,48 @@ fahrplan_gui_t::fahrplan_gui_t(schedule_t* fpl_, spieler_t* sp_, convoihandle_t
 	// set this schedule as current to show on minimap if possible
 	reliefkarte_t::get_karte()->set_current_fpl(fpl, sp->get_player_nr()); // (*fpl,player_nr)
 	set_resizemode(diagonal_resize);
+	update_selection();
 }
 
 
-
-
 void fahrplan_gui_t::update_werkzeug(bool set)
 {
-	karte_t *welt = sp->get_welt();
 	if(!set  ||  mode==removing  ||  mode==undefined_mode) {
 		// reset tools, if still selected ...
-		if(welt->get_werkzeug()==werkzeug_t::general_tool[WKZ_FAHRPLAN_ADD]) {
-			if(werkzeug_t::general_tool[WKZ_FAHRPLAN_ADD]->default_param==(const char *)fpl) {
-				welt->set_werkzeug( werkzeug_t::general_tool[WKZ_ABFRAGE] );
+		if(welt->get_werkzeug(sp->get_player_nr())==werkzeug_t::general_tool[WKZ_FAHRPLAN_ADD]) {
+			if(werkzeug_t::general_tool[WKZ_FAHRPLAN_ADD]->get_default_param()==(const char *)fpl) {
+				welt->set_werkzeug( werkzeug_t::general_tool[WKZ_ABFRAGE], sp );
 			}
 		}
-		else if(welt->get_werkzeug()==werkzeug_t::general_tool[WKZ_FAHRPLAN_INS]) {
-			if(werkzeug_t::general_tool[WKZ_FAHRPLAN_INS]->default_param==(const char *)fpl) {
-				welt->set_werkzeug( werkzeug_t::general_tool[WKZ_ABFRAGE] );
+		else if(welt->get_werkzeug(sp->get_player_nr())==werkzeug_t::general_tool[WKZ_FAHRPLAN_INS]) {
+			if(werkzeug_t::general_tool[WKZ_FAHRPLAN_INS]->get_default_param()==(const char *)fpl) {
+				welt->set_werkzeug( werkzeug_t::general_tool[WKZ_ABFRAGE], sp );
 			}
 		}
 	}
 	else {
 		//  .. or set them again
 		if(mode==adding) {
-			werkzeug_t::general_tool[WKZ_FAHRPLAN_ADD]->default_param = (const char *)fpl;
-			sp->get_welt()->set_werkzeug( werkzeug_t::general_tool[WKZ_FAHRPLAN_ADD] );
+			werkzeug_t::general_tool[WKZ_FAHRPLAN_ADD]->set_default_param((const char *)fpl);
+			sp->get_welt()->set_werkzeug( werkzeug_t::general_tool[WKZ_FAHRPLAN_ADD], sp );
 		}
 		else if(mode==inserting) {
-			werkzeug_t::general_tool[WKZ_FAHRPLAN_INS]->default_param = (const char *)fpl;
-			sp->get_welt()->set_werkzeug( werkzeug_t::general_tool[WKZ_FAHRPLAN_INS] );
+			werkzeug_t::general_tool[WKZ_FAHRPLAN_INS]->set_default_param((const char *)fpl);
+			sp->get_welt()->set_werkzeug( werkzeug_t::general_tool[WKZ_FAHRPLAN_INS], sp );
 		}
 	}
 }
 
 
-
-
 void fahrplan_gui_t::update_selection()
 {
 	// update load
 	lb_load.set_color( COL_GREY3 );
 	lb_wait.set_color( COL_GREY3 );
-	if(  fpl->get_count()>0  ) {
+	if (!fpl->empty()) {
 		fpl->set_aktuell( min(fpl->get_count()-1,fpl->get_aktuell()) );
 		const uint8 aktuell = fpl->get_aktuell();
-		if(haltestelle_t::get_halt(sp->get_welt(), fpl->eintrag[aktuell].pos).is_bound()) {
+		if(  haltestelle_t::get_halt(sp->get_welt(), fpl->eintrag[aktuell].pos, sp).is_bound()  ) {
 			lb_load.set_color( COL_BLACK );
 			numimp_load.set_value( fpl->eintrag[aktuell].ladegrad );
 			if(  fpl->eintrag[aktuell].ladegrad>0  ) {
@@ -354,17 +349,15 @@ void fahrplan_gui_t::update_selection()
 }
 
 
-
 /**
  * Mausklicks werden hiermit an die GUI-Komponenten
  * gemeldet
  */
-void
-fahrplan_gui_t::infowin_event(const event_t *ev)
+bool fahrplan_gui_t::infowin_event(const event_t *ev)
 {
 	if ( (ev)->ev_class == EVENT_CLICK  &&  !((ev)->ev_code==MOUSE_WHEELUP  ||  (ev)->ev_code==MOUSE_WHEELDOWN)  &&  !line_selector.getroffen(ev->cx, ev->cy-16))  {//  &&  !scrolly.getroffen(ev->cx, ev->cy+16)) {
 
-		// close combo box; we must do it ourselves, since the box does not recieve outside events ...
+		// close combo box; we must do it ourselves, since the box does not receive outside events ...
 		line_selector.close_box();
 
 		if(ev->my>=scrolly.get_pos().y+16) {
@@ -387,38 +380,47 @@ fahrplan_gui_t::infowin_event(const event_t *ev)
 			}
 		}
 	}
-	else if(ev->ev_class == INFOWIN  &&  ev->ev_code == WIN_CLOSE  ) {
+	else if(ev->ev_class == INFOWIN  &&  ev->ev_code == WIN_CLOSE  &&  fpl!=NULL  ) {
 
 		update_werkzeug( false );
 		fpl->cleanup();
 		fpl->eingabe_abschliessen();
-		if (cnv.is_bound()) {
-			// if a line is selected
-			if (new_line.is_bound()  &&  new_line->get_schedule()->matches( sp->get_welt(), fpl )) {
-				// if the selected line is different to the convoi's line, apply it
-				if(new_line!=cnv->get_line()) {
-					uint8 akt = fpl->get_aktuell();
-					cnv->set_line( new_line );
-					cnv->get_schedule()->set_aktuell( akt );
+		// now apply the changes
+		if(cnv.is_bound()) {
+			// do not send changes if the convoi is about to be deleted
+			if(  cnv->get_state() != convoi_t::SELF_DESTRUCT  ) {
+				// if a line is selected
+				if(  new_line.is_bound()  ) {
+					// if the selected line is different to the convoi's line, apply it
+					if(new_line!=cnv->get_line()) {
+						char id[16];
+						sprintf( id, "%i,%i", new_line.get_id(), fpl->get_aktuell() );
+						cnv->call_convoi_tool( 'l', id );
+					}
+					else {
+						cbuffer_t buf(5500);
+						fpl->sprintf_schedule( buf );
+						cnv->call_convoi_tool( 'g', buf );
+					}
+				}
+				else {
+					cbuffer_t buf(5500);
+					fpl->sprintf_schedule( buf );
+					cnv->call_convoi_tool( 'g', buf );
 				}
-			}
-			else {
-				// no line is selected or line does not match => unset the line
-				cnv->unset_line();
 			}
 		}
 	}
-	else if(ev->ev_class == INFOWIN  &&  (ev->ev_code == WIN_TOP  ||  ev->ev_code == WIN_OPEN)  ) {
+	else if(ev->ev_class == INFOWIN  &&  (ev->ev_code == WIN_TOP  ||  ev->ev_code == WIN_OPEN)  &&  fpl!=NULL  ) {
 		// just to be sure, renew the tools ...
 		update_werkzeug( true );
 	}
-	gui_frame_t::infowin_event(ev);
+
+	return gui_frame_t::infowin_event(ev);
 }
 
 
-
-bool
-fahrplan_gui_t::action_triggered( gui_action_creator_t *komp, value_t p)
+bool fahrplan_gui_t::action_triggered( gui_action_creator_t *komp, value_t p)
 {
 DBG_MESSAGE("fahrplan_gui_t::action_triggered()","komp=%p combo=%p",komp,&line_selector);
 
@@ -444,23 +446,26 @@ DBG_MESSAGE("fahrplan_gui_t::action_triggered()","komp=%p combo=%p",komp,&line_s
 		update_werkzeug( false );
 
 	} else if(komp == &numimp_load) {
-		if(fpl->get_count() > 0) {
+		if (!fpl->empty()) {
 			fpl->eintrag[fpl->get_aktuell()].ladegrad = p.i;
 			update_selection();
 		}
 	} else if(komp == &bt_wait_prev) {
-		if(fpl->get_count() > 0) {
+		if (!fpl->empty()) {
 			sint8& wait = fpl->eintrag[fpl->get_aktuell()].waiting_time_shift;
 			if(wait>7) {
 				wait --;
 			}
-			else {
+			else if(  wait>0  ) {
 				wait = 0;
 			}
+			else {
+				wait = 16;
+			}
 			update_selection();
 		}
 	} else if(komp == &bt_wait_next) {
-		if(fpl->get_count() > 0) {
+		if (!fpl->empty()) {
 			sint8& wait = fpl->eintrag[fpl->get_aktuell()].waiting_time_shift;
 			if(wait==0) {
 				wait = 7;
@@ -468,6 +473,9 @@ DBG_MESSAGE("fahrplan_gui_t::action_triggered()","komp=%p combo=%p",komp,&line_s
 			else if(wait<16) {
 				wait ++;
 			}
+			else {
+				wait = 0;
+			}
 			update_selection();
 		}
 	} else if (komp == &bt_return) {
@@ -486,20 +494,30 @@ DBG_MESSAGE("fahrplan_gui_t::action_triggered()","komp=%p combo=%p",komp,&line_s
 			line_selector.set_selection( 0 );
 		}
 	} else if (komp == &bt_promote_to_line) {
-		this->fpl->eingabe_abschliessen();
-		new_line = sp->simlinemgmt.create_line(fpl->get_type(), sp, this->fpl);
-		this->fpl->eingabe_beginnen();
-		init_line_selector();
+		// update line schedule via tool!
+		werkzeug_t *w = create_tool( WKZ_LINE_TOOL | SIMPLE_TOOL );
+		cbuffer_t buf(5500);
+		buf.printf( "c,0,%i,%ld,", (int)fpl->get_type(), (long)old_fpl );
+		fpl->sprintf_schedule( buf );
+		w->set_default_param(buf);
+		sp->get_welt()->set_werkzeug( w, sp );
+		// since init always returns false, it is save to delete immediately
+		delete w;
 	}
+	scrolly.set_groesse( scrolly.get_groesse() );
 	// recheck lines
 	if (cnv.is_bound()) {
 		// unequal to line => remove from line ...
-		if(new_line.is_bound()  &&   !fpl->matches(sp->get_welt(),new_line->get_schedule())) {
+		if(  new_line.is_bound()  &&  !fpl->matches(sp->get_welt(),new_line->get_schedule())  ) {
 			new_line = linehandle_t();
 			line_selector.set_selection(0);
 		}
+		// only assing old line, when new_line is not equal
+		if(  !new_line.is_bound()  &&  old_line.is_bound()  &&   fpl->matches(sp->get_welt(),old_line->get_schedule())  ) {
+			new_line = old_line;
+			init_line_selector();
+		}
 	}
-	scrolly.set_groesse( scrolly.get_groesse() );
 	return true;
 }
 
@@ -509,40 +527,165 @@ void fahrplan_gui_t::init_line_selector()
 	line_selector.clear_elements();
 	line_selector.append_element( new gui_scrolled_list_t::const_text_scrollitem_t( no_line, COL_BLACK ) );
 	int selection = -1;
+	sp->simlinemgmt.sort_lines();	// to take care of renaming ...
 	sp->simlinemgmt.get_lines(fpl->get_type(), &lines);
+
+	// keep assignment with identical schedules
+	if(  new_line.is_bound()  &&  !fpl->matches( sp->get_welt(), new_line->get_schedule() )  ) {
+		if(  old_line.is_bound()  &&  fpl->matches( sp->get_welt(), old_line->get_schedule() )  ) {
+			new_line = old_line;
+		}
+		else {
+			new_line = linehandle_t();
+		}
+	}
+
 	for (vector_tpl<linehandle_t>::const_iterator i = lines.begin(), end = lines.end(); i != end; i++) {
 		linehandle_t line = *i;
 		line_selector.append_element( new line_scrollitem_t(line) );
-		if (new_line == line) {
+		if(  !new_line.is_bound()  ) {
+			if(  fpl->matches( sp->get_welt(), line->get_schedule() )  ) {
+				selection = line_selector.count_elements() - 1;
+				new_line = line;
+			}
+		}
+		else if(  line==new_line  ) {
 			selection = line_selector.count_elements() - 1;
 		}
 	}
 	line_selector.set_selection( selection );
+	old_line_count = sp->simlinemgmt.get_line_count();
+	last_schedule_count = fpl->get_count();
 }
 
 
 
 void fahrplan_gui_t::zeichnen(koord pos, koord gr)
 {
-	if(  new_line.is_bound()  &&  !new_line->get_schedule()->matches(sp->get_welt(),fpl)  ) {
-		line_selector.set_selection(0);
+	if(  sp->simlinemgmt.get_line_count()!=old_line_count  ||  last_schedule_count!=fpl->get_count()  ) {
+		// lines added or deleted
+		init_line_selector();
+		last_schedule_count = fpl->get_count();
 	}
+
+	// after loading in network games, the schedule might still being updated
+	if(  cnv.is_bound()  &&  cnv->get_state()==convoi_t::FAHRPLANEINGABE  &&  fpl->ist_abgeschlossen()  ) {
+		assert( convoi_t::FAHRPLANEINGABE==1 ); // convoi_t::FAHRPLANEINGABE is 1
+		fpl->eingabe_beginnen();
+		cnv->call_convoi_tool( 's', "1" );
+	}
+
 	gui_frame_t::zeichnen(pos,gr);
 }
 
 
 
 /**
- * resize window in response to a resize event
+ * Set window size and adjust component sizes and/or positions accordingly
  * @author Hj. Malthaner
  * @date   16-Oct-2003
  */
-void fahrplan_gui_t::resize(const koord delta)
+void fahrplan_gui_t::set_fenstergroesse(koord groesse)
 {
-	gui_frame_t::resize(delta);
+	gui_frame_t::set_fenstergroesse(groesse);
 
-	const koord groesse = get_fenstergroesse();
+	groesse = get_fenstergroesse();		// may subject to min window size limitation
 	scrolly.set_groesse( koord(groesse.x, groesse.y-scrolly.get_pos().y-16) );
 
 	line_selector.set_max_size(koord(BUTTON_WIDTH*3, groesse.y-line_selector.get_pos().y -2*16));
 }
+
+
+void fahrplan_gui_t::map_rotate90( sint16 y_size)
+{
+	fpl->rotate90(y_size);
+}
+
+
+fahrplan_gui_t::fahrplan_gui_t(karte_t *welt):
+	gui_frame_t("Fahrplan", NULL),
+	lb_line("Serves Line:"),
+	lb_wait("month wait time"),
+	lb_waitlevel(NULL, COL_WHITE, gui_label_t::right),
+	lb_load("Full load"),
+	stats(welt,NULL),
+	scrolly(&stats),
+	fpl(NULL),
+	old_fpl(NULL),
+	sp(NULL),
+	cnv()
+{
+	// just a dummy
+	this->welt = welt;
+}
+
+
+void fahrplan_gui_t::rdwr(loadsave_t *file)
+{
+	// this handles only schedules of bound convois
+	// lines are handled by line_management_gui_t
+	koord3d cnv_pos;
+	char cnv_name[256];
+	uint8 player_nr;
+	koord gr = get_fenstergroesse();
+	if(  file->is_saving()  ) {
+		tstrncpy( cnv_name, cnv->get_name(), sizeof(cnv_name) );
+		player_nr = sp->get_player_nr();
+		cnv_pos = cnv->get_pos();
+	}
+	else {
+		// dummy types
+		old_fpl = new autofahrplan_t();
+		fpl = new autofahrplan_t();
+	}
+	gr.rdwr( file );
+	file->rdwr_byte( player_nr );
+	file->rdwr_str( cnv_name, sizeof(cnv_name) );
+	cnv_pos.rdwr( file );
+	old_fpl->rdwr(file);
+	fpl->rdwr(file);
+	if(  file->is_loading()  ) {
+		// find convoi by name and position (since there could be two convois with the same name)
+		if(  grund_t *gr = welt->lookup(cnv_pos)  ) {
+			for(  uint8 i=0;  i<gr->get_top();  i++  ) {
+				if(  gr->obj_bei(i)->is_moving()  ) {
+					vehikel_t const* const v = ding_cast<vehikel_t>(gr->obj_bei(i));
+					if(  v  &&  v->get_besitzer()->get_player_nr()==player_nr  &&  v->get_convoi()  &&  strcmp(v->get_convoi()->get_name(),cnv_name)==0  &&  old_fpl->matches( welt, v->get_convoi()->get_schedule() )  ) {
+						cnv = v->get_convoi()->self;
+						break;
+					}
+				}
+			}
+		}
+		if(  !cnv.is_bound() ) {
+			// not found (most likely convoi in depot ... )
+			for(  vector_tpl<convoihandle_t>::const_iterator i=welt->convois_begin(); i!=welt->convois_end();  ++i  ) {
+				if(  (*i)->get_besitzer()->get_player_nr()==player_nr  &&  strncmp( (*i)->get_name(), cnv_name, 256 )==0  &&  old_fpl->matches( welt, (*i)->get_schedule() )  ) {
+					// valid convoi found
+					cnv = *i;
+					break;
+				}
+			}
+		}
+		if(  cnv.is_bound() ) {
+			// now we can open the window ...
+			KOORD_VAL xpos = win_get_posx( this );
+			KOORD_VAL ypos = win_get_posy( this );
+			fahrplan_gui_t *w = new fahrplan_gui_t( cnv->get_schedule(), cnv->get_besitzer(), cnv );
+			create_win( xpos, ypos, w, w_info, (long)cnv->get_schedule() );
+			w->set_fenstergroesse( gr );
+			w->fpl->copy_from( fpl );
+			cnv->get_schedule()->eingabe_abschliessen();
+			w->fpl->eingabe_abschliessen();
+		}
+		else {
+			dbg->error( "fahrplan_gui_t::rdwr", "Could not restore schedule window for %s", cnv_name );
+		}
+		sp = NULL;
+		delete old_fpl;
+		delete fpl;
+		fpl = old_fpl = NULL;
+		cnv = convoihandle_t();
+		destroy_win( this );
+	}
+}
