@@ -37,7 +37,7 @@ public:
 	 * @return den Dateinamen fr die Hilfe, oder NULL
 	 * @author Hj. Malthaner
 	 */
-	const char * gib_hilfe_datei() const { return "color.txt"; }
+	const char * get_hilfe_datei() const { return "color.txt"; }
 
 //	void infowin_event(const event_t *ev);
 
