@@ -5,7 +5,7 @@
  * (see licence.txt)
  */
 
-#include <string.h>
+#include <string>
 #include <stdio.h>
 
 #include "../simio.h"
@@ -21,15 +21,16 @@
  */
 void load_relief_frame_t::action(const char *filename)
 {
-	cstring_t p("maps/");
+	std::string p("maps/");
 	sets->heightfield = p+filename;
 }
 
 
-void load_relief_frame_t::del_action(const char *filename)
+bool load_relief_frame_t::del_action(const char *filename)
 {
-	cstring_t p("maps/");
-	remove(p+filename);
+	std::string p("maps/");
+	remove((p+filename).c_str());
+	return false;
 }
 
 
@@ -70,5 +71,3 @@ bool load_relief_frame_t::check_file( const char *filename, const char * )
 	}
 	return false;
 }
-
-
