@@ -9,9 +9,11 @@
  * [Mathew Hounsell] Min Size Button On Map Window 20030313
  */
 
+#include <string>
 #include <stdio.h>
 #include <cmath>
 
+#include "karte.h"
 #include "map_frame.h"
 #include "components/list_button.h"
 
@@ -20,19 +22,20 @@
 #include "../simgraph.h"
 #include "../simcolor.h"
 #include "../bauer/fabrikbauer.h"
-#include "../utils/cstring_t.h"
 #include "../dataobj/umgebung.h"
 #include "../dataobj/translator.h"
 #include "../dataobj/koord.h"
+#include "../dataobj/loadsave.h"
 #include "../besch/fabrik_besch.h"
 
 
 static koord old_ij=koord::invalid;
 
 koord map_frame_t::size=koord(0,0);
-uint8 map_frame_t::legend_visible=false;
-uint8 map_frame_t::scale_visible=false;
-uint8 map_frame_t::directory_visible=false;
+bool map_frame_t::legend_visible=false;
+bool map_frame_t::scale_visible=false;
+bool map_frame_t::directory_visible=false;
+bool map_frame_t::is_cursor_hidden=false;
 
 // Hajo: we track our position onscreen
 koord map_frame_t::screenpos;
@@ -41,9 +44,9 @@ koord map_frame_t::screenpos;
 struct legend_entry
 {
 	legend_entry() {}
-	legend_entry(const cstring_t& text_, int colour_) : text(text_), colour(colour_) {}
+	legend_entry(const std::string &text_, int colour_) : text(text_), colour(colour_) {}
 
-	cstring_t text;
+	std::string text;
 	int       colour;
 };
 
@@ -70,16 +73,17 @@ const char map_frame_t::map_type[MAX_BUTTON_TYPE][64] =
 	"Factories",
 	"Depots",
 	"Forest",
-	"CityLimit"
+	"CityLimit",
+	"PaxDest"
 };
 
 const uint8 map_frame_t::map_type_color[MAX_BUTTON_TYPE] =
 {
-	215, 23, 31, 157, 46, 55, 63, 133, 79, 191, 207, 11, 123, 221, 71, 135, 127, 198
+	215, 23, 31, 157, 46, 55, 63, 133, 79, 191, 207, 11, 123, 221, 71, 135, 127, 198, 23
 };
 
 
-map_frame_t::map_frame_t(const karte_t *welt) :
+map_frame_t::map_frame_t(karte_t *welt) :
 	gui_frame_t("Reliefkarte"),
 	scrolly(reliefkarte_t::get_karte()),
 	zoom_label("map zoom")
@@ -97,18 +101,18 @@ map_frame_t::map_frame_t(const karte_t *welt) :
 	add_komponente( &zoom_label );
 
 	// rotate map 45
-	b_rotate45.init(button_t::square, "isometric map", koord(BUTTON_WIDTH+40,BUTTON_HEIGHT+4), koord(BUTTON_WIDTH,BUTTON_HEIGHT));
+	b_rotate45.init(button_t::square, "isometric map", koord(BUTTON_WIDTH+40,BUTTON_HEIGHT+4));
 	b_rotate45.add_listener(this);
 	add_komponente(&b_rotate45);
 
 	// show/hide schedule
-	b_show_schedule.init(button_t::square, "Show schedules", koord(BUTTON_WIDTH+40,BUTTON_HEIGHT*2+4), koord(BUTTON_WIDTH,BUTTON_HEIGHT)); // right align
+	b_show_schedule.init(button_t::square, "Show schedules", koord(BUTTON_WIDTH+40,BUTTON_HEIGHT*2+4)); // right align
 	b_show_schedule.set_tooltip("Shows the currently selected schedule");
 	b_show_schedule.add_listener(this);
 	add_komponente( &b_show_schedule );
 
 	// show/hide schedule
-	b_show_fab_connections.init(button_t::square, "factory details", koord(2,BUTTON_HEIGHT*2+4), koord(BUTTON_WIDTH,BUTTON_HEIGHT)); // right align
+	b_show_fab_connections.init(button_t::square, "factory details", koord(2,BUTTON_HEIGHT*2+4)); // right align
 	b_show_fab_connections.set_tooltip("Shows consumer/suppliers for factories");
 	b_show_fab_connections.add_listener(this);
 	add_komponente( &b_show_fab_connections );
@@ -132,15 +136,14 @@ map_frame_t::map_frame_t(const karte_t *welt) :
 	// add factory names; shorten too long names
 	while(iter.next()) {
 		if(iter.get_current_value()->get_gewichtung()>0) {
-			int i;
+			size_t i;
 
-			cstring_t label (translator::translate(iter.get_current_value()->get_name()));
-			for(  i=12;  i<label.len()  &&  display_calc_proportional_string_len_width(label,i)<100;  i++  )
+			std::string label (translator::translate(iter.get_current_value()->get_name()));
+			for(  i=12;  i<label.size()  &&  display_calc_proportional_string_len_width(label.c_str(),i)<100;  i++  )
 				;
-			if(  i<label.len()  ) {
-				label.set_at(i++, '.');
-				label.set_at(i++, '.');
-				label.set_at(i++, '\0');
+			if(  i<label.size()  ) {
+				label = label.substr(0, i);
+				label.append("..");
 			}
 
 			legend.append(legend_entry(label, iter.get_current_value()->get_kennfarbe()));
@@ -173,30 +176,77 @@ map_frame_t::map_frame_t(const karte_t *welt) :
 	set_fenstergroesse(size);
 	resize( koord(0,0) );
 
-	// Clipping gendert - max. 250 war zu knapp fr grosse Karten - V.Meyer
 	reliefkarte_t *karte = reliefkarte_t::get_karte();
+	karte->set_welt( welt );
+
 	const koord gr = karte->get_groesse();
 	const koord s_gr=scrolly.get_groesse();
 	const koord ij = welt->get_world_position();
 	const koord win_size = gr-s_gr;	// this is the visible area
 	scrolly.set_scroll_position(  max(0,min(ij.x-win_size.x/2,gr.x)), max(0, min(ij.y-win_size.y/2,gr.y)) );
 
+	old_ij = koord::invalid;
+
 	// Hajo: Trigger layouting
 	set_resizemode(diagonal_resize);
 
 	is_dragging = false;
+	zoomed = false;
 
 	karte->set_mode( (reliefkarte_t::MAP_MODES)umgebung_t::default_mapmode );
 }
 
 
+void map_frame_t::rdwr( loadsave_t *file )
+{
+	file->rdwr_bool( reliefkarte_t::get_karte()->rotate45 );
+	file->rdwr_bool( reliefkarte_t::get_karte()->is_show_schedule );
+	file->rdwr_bool( reliefkarte_t::get_karte()->is_show_fab );
+	file->rdwr_short( reliefkarte_t::get_karte()->zoom_in );
+	file->rdwr_short( reliefkarte_t::get_karte()->zoom_out );
+	bool show_legend_state = legend_visible;
+	file->rdwr_bool( show_legend_state );
+	file->rdwr_bool( scale_visible );
+	file->rdwr_bool( directory_visible );
+	file->rdwr_byte( umgebung_t::default_mapmode );
+
+	if(  file->is_loading()  ) {
+		koord savesize;
+		savesize.rdwr(file);
+		set_fenstergroesse( savesize );
+		resize( koord(0,0) );
+		// notify minimap of new settings
+		reliefkarte_t::get_karte()->calc_map_groesse();
+		scrolly.set_groesse( scrolly.get_groesse() );
+
+		sint32 xoff;
+		file->rdwr_long( xoff );
+		sint32 yoff;
+		file->rdwr_long( yoff );
+		scrolly.set_scroll_position( xoff, yoff );
+
+		reliefkarte_t::get_karte()->set_mode((reliefkarte_t::MAP_MODES)umgebung_t::default_mapmode);
+		for (int i=0;i<MAX_BUTTON_TYPE;i++) {
+			filter_buttons[i].pressed = i==umgebung_t::default_mapmode;
+		}
+		if(  legend_visible!=show_legend_state  ) {
+			action_triggered( &b_show_legend, (long)0 );
+		}
+	}
+	else {
+		koord gr = get_fenstergroesse();
+		gr.rdwr(file);
+		sint32 xoff = scrolly.get_scroll_x();
+		file->rdwr_long( xoff );
+		sint32 yoff = scrolly.get_scroll_y();
+		file->rdwr_long( yoff );
+	}
+}
+
 
 // button pressed
-bool
-map_frame_t::action_triggered( gui_action_creator_t *komp,value_t /* */)
+bool map_frame_t::action_triggered( gui_action_creator_t *komp,value_t /* */)
 {
-	reliefkarte_t::get_karte()->calc_map();
-
 	if(komp==&b_show_legend) {
 		if(!legend_visible) {
 			for (int type=0; type<MAX_BUTTON_TYPE; type++) {
@@ -223,32 +273,16 @@ map_frame_t::action_triggered( gui_action_creator_t *komp,value_t /* */)
 	}
 	else if(komp==zoom_buttons+1) {
 		// zoom out
-		if(reliefkarte_t::get_karte()->zoom_in>1) {
-			reliefkarte_t::get_karte()->zoom_in--;
-		}
-		else if(reliefkarte_t::get_karte()->zoom_out<4) {
-			reliefkarte_t::get_karte()->zoom_out++;
-		}
-		reliefkarte_t::get_karte()->calc_map();
-		// recalc sliders
-		scrolly.set_groesse( scrolly.get_groesse() );
+		zoom(true);
 	}
 	else if(komp==zoom_buttons+0) {
 		// zoom in
-		if(reliefkarte_t::get_karte()->zoom_out>1) {
-			reliefkarte_t::get_karte()->zoom_out--;
-		}
-		else if(reliefkarte_t::get_karte()->zoom_in<4) {
-			reliefkarte_t::get_karte()->zoom_in++;
-		}
-		reliefkarte_t::get_karte()->calc_map();
-		// recalc sliders
-		scrolly.set_groesse( scrolly.get_groesse() );
+		zoom(false);
 	}
 	else if(komp==&b_rotate45) {
 		// rotated/straight map
 		reliefkarte_t::get_karte()->rotate45 ^= 1;
-		reliefkarte_t::get_karte()->calc_map();
+		reliefkarte_t::get_karte()->calc_map_groesse();
 		scrolly.set_groesse( scrolly.get_groesse() );
 	}
 	else if(komp==&b_show_schedule) {
@@ -280,55 +314,83 @@ map_frame_t::action_triggered( gui_action_creator_t *komp,value_t /* */)
 }
 
 
+void map_frame_t::zoom(bool zoom_out)
+{
+	if (zoom_out) {
+		// zoom out
+		if(reliefkarte_t::get_karte()->zoom_in>1) {
+			reliefkarte_t::get_karte()->zoom_in--;
+		}
+		else if(reliefkarte_t::get_karte()->zoom_out<4  ) {
+			reliefkarte_t::get_karte()->zoom_out++;
+		}
+	}
+	else {
+		// zoom in
+		if(reliefkarte_t::get_karte()->zoom_out>1) {
+			reliefkarte_t::get_karte()->zoom_out--;
+		}
+		else if(reliefkarte_t::get_karte()->zoom_in<8  ) {
+			reliefkarte_t::get_karte()->zoom_in++;
+		}
+	}
+	// recalc map size
+	reliefkarte_t::get_karte()->calc_map_groesse();
+	// recalc all the other data incl scrollbars
+	resize(koord(0,0));
+
+	zoomed = true;
+}
+
 
 /**
  * Events werden hiermit an die GUI-Komponenten
  * gemeldet
  * @author Hj. Malthaner
  */
-void map_frame_t::infowin_event(const event_t *ev)
+bool map_frame_t::infowin_event(const event_t *ev)
 {
 	if(ev->ev_class == INFOWIN) {
 		if(ev->ev_code == WIN_OPEN) {
-			reliefkarte_t::get_karte()->is_visible = true;
-			reliefkarte_t::get_karte()->calc_map();
+			reliefkarte_t::get_karte()->set_xy_offset_size( koord(0,0), koord(0,0) );
 		}
 		else if(ev->ev_code == WIN_CLOSE) {
 			reliefkarte_t::get_karte()->is_visible = false;
 		}
 	}
 
-	if(  (IS_WHEELUP(ev) || IS_WHEELDOWN(ev))  &&  reliefkarte_t::get_karte()->getroffen(ev->mx,ev->my)) {
+	if(  (IS_WHEELUP(ev) || IS_WHEELDOWN(ev))  &&  reliefkarte_t::get_karte()->getroffen(ev->mx-scrolly.get_pos().x,ev->my-scrolly.get_pos().y-16)) {
 		// otherwise these would go to the vertical scroll bar
-		reliefkarte_t::get_karte()->infowin_event(ev);
-		return;
-	}
-
-	if(!is_dragging) {
-		gui_frame_t::infowin_event(ev);
+		zoom(IS_WHEELUP(ev));
+		return true;
 	}
 
 	// Hajo: hack: relief map can resize upon right click
 	// we track this here, and adjust size.
-	if(IS_RIGHTCLICK(ev)) {
+	if(  IS_RIGHTCLICK(ev)  ) {
 		is_dragging = false;
+		display_show_pointer(false);
+		is_cursor_hidden = true;
 		reliefkarte_t::get_karte()->get_welt()->set_scroll_lock(false);
+		return true;
 	}
-
-	if(IS_RIGHTRELEASE(ev)) {
+	else if(  IS_RIGHTRELEASE(ev)  ) {
 		if(!is_dragging) {
 			resize( koord(0,0) );
 		}
 		is_dragging = false;
+		display_show_pointer(true);
+		is_cursor_hidden = false;
 		reliefkarte_t::get_karte()->get_welt()->set_scroll_lock(false);
+		return true;
 	}
-
-	if(reliefkarte_t::get_karte()->getroffen(ev->mx,ev->my)  &&  IS_RIGHTDRAG(ev)) {
+	else if(  IS_RIGHTDRAG(ev)  &&  reliefkarte_t::get_karte()->getroffen(ev->mx,ev->my)  ) {
 		int x = scrolly.get_scroll_x();
 		int y = scrolly.get_scroll_y();
+		const int scroll_direction = ( umgebung_t::scroll_multi>0 ? 1 : -1 );
 
-		x += (ev->mx - ev->cx)*2;
-		y += (ev->my - ev->cy)*2;
+		x += (ev->mx - ev->cx)*scroll_direction*2;
+		y += (ev->my - ev->cy)*scroll_direction*2;
 
 		is_dragging = true;
 		reliefkarte_t::get_karte()->get_welt()->set_scroll_lock(true);
@@ -337,9 +399,16 @@ void map_frame_t::infowin_event(const event_t *ev)
 
 		// Hajo: re-center mouse pointer
 		display_move_pointer(screenpos.x+ev->cx, screenpos.y+ev->cy);
+		return true;
+	}
+	else if(  is_cursor_hidden  )
+	{
+		display_show_pointer(true);
+		is_cursor_hidden = false;
 	}
-}
 
+	return gui_frame_t::infowin_event(ev);
+}
 
 
 /**
@@ -358,7 +427,6 @@ DBG_MESSAGE("map_frame_t::set_fenstergroesse()","gr.x=%i, gr.y=%i",size.x,size.y
 }
 
 
-
 /**
  * resize window in response to a resize event
  * @author Hj. Malthaner
@@ -366,7 +434,7 @@ DBG_MESSAGE("map_frame_t::set_fenstergroesse()","gr.x=%i, gr.y=%i",size.x,size.y
  */
 void map_frame_t::resize(const koord delta)
 {
-	karte_t *welt=reliefkarte_t::get_karte()->get_welt();
+	karte_t *welt = reliefkarte_t::get_karte()->get_welt();
 
 	koord groesse = get_fenstergroesse()+delta;
 
@@ -422,13 +490,10 @@ void map_frame_t::resize(const koord delta)
 	if(offset_y>groesse.y) {
 		groesse.y = offset_y;
 	}
-	old_ij = koord::invalid;
-
 	set_fenstergroesse( groesse );
 }
 
 
-
 /**
  * komponente neu zeichnen. Die bergebenen Werte beziehen sich auf
  * das Fenster, d.h. es sind die Bildschirkoordinaten des Fensters
@@ -440,22 +505,24 @@ void map_frame_t::zeichnen(koord pos, koord gr)
 	// update our stored screen position
 	screenpos = pos;
 
+	reliefkarte_t::get_karte()->set_xy_offset_size( koord(scrolly.get_scroll_x(), scrolly.get_scroll_y()), koord(scrolly.get_groesse()-koord(12,11)) );
+
 	// first: check if cursor within map screen size
 	karte_t *welt=reliefkarte_t::get_karte()->get_welt();
 	koord ij = welt->get_world_position();
 	if(welt->ist_in_kartengrenzen(ij)) {
 		reliefkarte_t::get_karte()->karte_to_screen(ij);
-		// only recenter by zoom or position change; we want still be able to scroll
-		if(old_ij!=ij) {
-			koord groesse = scrolly.get_groesse();
-			if(
-				(scrolly.get_scroll_x()>ij.x  ||  scrolly.get_scroll_x()+groesse.x<=ij.x) ||
-				(scrolly.get_scroll_y()>ij.y  ||  scrolly.get_scroll_y()+groesse.y<=ij.y) ) {
+		// only recenter if zoomed or world position has changed and its outside visible area
+		const koord groesse = scrolly.get_groesse();
+		if(zoomed  ||  ( old_ij != ij  &&
+				( scrolly.get_scroll_x()>ij.x  ||  scrolly.get_scroll_x()+groesse.x<=ij.x  ||
+				  scrolly.get_scroll_y()>ij.y  ||  scrolly.get_scroll_y()+groesse.y<=ij.y ) ) ) {
 				// recenter cursor by scrolling
 				scrolly.set_scroll_position( max(0,ij.x-(groesse.x/2)), max(0,ij.y-(groesse.y/2)) );
-				old_ij = ij;
-			}
+				zoomed = false;
 		}
+		// remember world position, we do not want to have surprises when scrolling later on
+		old_ij = ij;
 	}
 
 	b_rotate45.pressed = reliefkarte_t::get_karte()->rotate45;
@@ -502,7 +569,7 @@ void map_frame_t::zeichnen(koord pos, koord gr)
 				break;
 			}
 			display_fillbox_wh(xpos, ypos+ 1 , 7, 7, i->colour, false);
-			display_proportional(xpos + 8, ypos, i->text, ALIGN_LEFT, COL_BLACK, false);
+			display_proportional(xpos + 8, ypos, i->text.c_str(), ALIGN_LEFT, COL_BLACK, false);
 		}
 	}
 }
