@@ -12,9 +12,10 @@
 #include "gui_frame.h"
 #include "components/gui_button.h"
 #include "components/gui_scrollpane.h"
+#include "components/gui_tab_panel.h"
+#include "components/gui_textinput.h"
 
 #include "message_stats_t.h"
-#include "../simmesg.h"
 #include "components/action_listener.h"
 
 class karte_t;
@@ -27,9 +28,12 @@ class karte_t;
 class message_frame_t : public gui_frame_t, private action_listener_t
 {
 private:
+	char ibuf[256];
 	message_stats_t	stats;
 	gui_scrollpane_t scrolly;
-	button_t option_bt;
+	gui_tab_panel_t tabs;		// Knightly : tab panel for filtering messages
+	gui_textinput_t input;
+	button_t option_bt, send_bt;
 	static karte_t *welt;
 
 public:
