@@ -2,9 +2,9 @@
 #define gui_messagebox_h
 
 #include "gui_frame.h"
-#include "components/gui_world_view_t.h"
+#include "components/location_view_t.h"
 #include "components/gui_image.h"
-#include "components/gui_textarea.h"
+#include "components/gui_fixedwidth_textarea.h"
 #include "../simskin.h"
 #include "../besch/skin_besch.h"
 #include "../simcolor.h"
@@ -18,14 +18,14 @@ class news_window : public gui_frame_t
 public:
 	virtual PLAYER_COLOR_VAL get_titelcolor() const { return color; }
 
-	virtual ~news_window();
-
 protected:
 	news_window(const char* text, PLAYER_COLOR_VAL color);
 
+	// Knightly : to extend the window with an extra component in the upper right corner
+	void extend_window_with_component(gui_komponente_t *const component, const koord size, const koord offset = koord(0,0));
+
 private:
-	char *text;
-	gui_textarea_t meldung;
+	gui_fixedwidth_textarea_t textarea;
 	PLAYER_COLOR_VAL color;
 };
 
@@ -34,7 +34,7 @@ private:
 class news_img : public news_window
 {
 public:
-	news_img(const char* text, image_id bild = skinverwaltung_t::meldungsymbol->gib_bild_nr(0), PLAYER_COLOR_VAL color = WIN_TITEL);
+	news_img(const char* text, image_id bild = skinverwaltung_t::meldungsymbol->get_bild_nr(0), PLAYER_COLOR_VAL color = WIN_TITEL);
 
 private:
 	gui_image_t bild;
@@ -50,7 +50,7 @@ public:
 	void map_rotate90( sint16 new_ysize );
 
 private:
-	world_view_t view;
+	location_view_t view;
 };
 
 #endif
