@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997 - 2001 Hansjrg Malthaner
+ * Copyright (c) 1997 - 2001 Hansjörg Malthaner
  *
  * This file is part of the Simutrans project under the artistic licence.
  * (see licence.txt)
@@ -12,6 +12,9 @@
 #include <io.h>
 #include <direct.h>
 #endif
+
+#include <string>
+
 #include <sys/stat.h>
 #include <string.h>
 #include <time.h>
@@ -21,6 +24,8 @@
 #include "../simdebug.h"
 #include "../simwin.h"
 #include "../simintr.h"
+
+#include "../dataobj/umgebung.h"
 #include "../utils/simstring.h"
 
 #include "components/list_button.h"
@@ -33,13 +38,15 @@
 // Since during initialisations virtual functions do not work yet
 // in derived classes (since the object in question is not full initialized yet)
 // this functions returns true for files to be added.
-savegame_frame_t::savegame_frame_t(const char *suffix, const char *path ) :
-	gui_frame_t("Load/Save") ,
+savegame_frame_t::savegame_frame_t(const char *suffix, const char *path, bool only_directories ) :
+	gui_frame_t("Load/Save"),
+	input(),
 	fnlabel("Filename"),
 	scrolly(&button_frame)
 {
 	this->suffix = suffix;
 	this->fullpath = path;
+	this->only_directories = only_directories;
 	use_pak_extension = suffix==NULL  ||  strcmp( suffix, ".sve" )==0;
 	in_action = false;
 
@@ -52,7 +59,6 @@ savegame_frame_t::savegame_frame_t(const char *suffix, const char *path ) :
 	// Input box for game name
 	tstrncpy(ibuf, "", lengthof(ibuf));
 	input.set_text(ibuf, 128);
-	input.add_listener(this);
 	input.set_pos(koord(75,8));
 	input.set_groesse(koord(DIALOG_WIDTH-75-10-10, 14));
 	add_komponente(&input);
@@ -88,6 +94,7 @@ savegame_frame_t::savegame_frame_t(const char *suffix, const char *path ) :
 	cancelbutton.add_listener(this);
 	add_komponente(&cancelbutton);
 
+	set_focus( &input );
 	set_fenstergroesse(koord(DIALOG_WIDTH, y + 40));
 }
 
@@ -105,7 +112,7 @@ void savegame_frame_t::fill_list()
 #else
 		sprintf( searchpath, "%s/*%s", SAVE_PATH, suffix==NULL ? "" : suffix );
 #endif
-#ifndef	WIN32
+#ifndef	_WIN32
 		mkdir(SAVE_PATH, 0700);
 #else
 		mkdir(SAVE_PATH);
@@ -147,19 +154,25 @@ void savegame_frame_t::fill_list()
 #else
 	{
 		struct _finddata_t entry;
-		long hfind;
+		size_t hfind;
 
-		hfind = _findfirst( searchpath, &entry);
+		hfind = _findfirst( searchpath, &entry );
 		if(hfind == -1) {
 			dbg->warning("savegame_frame_t::savegame_frame_t()","Couldn't read directory.");
 		}
 		else {
 			do {
+				if(only_directories) {
+					if ((entry.attrib & _A_SUBDIR)==0) {
+						continue;
+					}
+				}
 				if(check_file(entry.name,suffix)) {
 					add_file(entry.name, get_info(entry.name), not_cutting_extension);
 				}
 			} while(_findnext(hfind, &entry) == 0 );
 		}
+		_findclose(hfind);
 	}
 #endif
 
@@ -212,7 +225,7 @@ savegame_frame_t::~savegame_frame_t()
 // sets the current filename in the input box
 void savegame_frame_t::set_filename(const char *fn)
 {
-	long len = strlen(fn);
+	size_t len = strlen(fn);
 	if(len>=4  &&  len-SAVE_PATH_X_LEN-3<128) {
 		if(strncmp(fn,SAVE_PATH_X,SAVE_PATH_X_LEN)==0) {
 			tstrncpy(ibuf, fn+SAVE_PATH_X_LEN, len-SAVE_PATH_X_LEN-3 );
@@ -232,19 +245,46 @@ void savegame_frame_t::add_file(const char *filename, const char *pak, const boo
 	char * name = new char [strlen(filename)+10];
 	char * date = new char[strlen(pak)+1];
 
-	sprintf(date, "%s", pak);
-	sprintf(name, "%s", filename);
+	strcpy( date, pak );
+	strcpy( name, filename );
 	if(!no_cutting_suffix) {
 		name[strlen(name)-4] = '\0';
 	}
 	button->set_no_translate(true);
 	button->set_text(name);	// to avoid translation
 
+	const std::string compare_to = umgebung_t::objfilename.size()>0  ?  umgebung_t::objfilename.substr( 0, umgebung_t::objfilename.size()-1 ) + " -"  :  std::string();
 	// sort by date descending:
 	slist_tpl<entry>::iterator i = entries.begin();
-	for (slist_tpl<entry>::iterator end = entries.end(); i != end; ++i) {
-		if (strcmp(i->label->get_text_pointer(), date) < 0) {
-			break;
+	slist_tpl<entry>::iterator end = entries.end();
+	if(  strncmp( compare_to.c_str(), pak, compare_to.size() )!=0  ) {
+		// skip current ones
+		while(  i != end  ) {
+			// extract palname in same format than in savegames ...
+			if(  strncmp( compare_to.c_str(), i->label->get_text_pointer(), compare_to.size() ) !=0  ) {
+				break;
+			}
+			++i;
+		}
+		// now just sort according time
+		while(  i != end  ) {
+			if(  strcmp(i->label->get_text_pointer(), date) < 0  ) {
+				break;
+			}
+			++i;
+		}
+	}
+	else {
+		// Insert to our games (or in front if none)
+		while(  i != end  ) {
+			if(  strcmp(i->label->get_text_pointer(), date) < 0  ) {
+				break;
+			}
+			// not our savegame any more => insert
+			if(  strncmp( compare_to.c_str(), i->label->get_text_pointer(), compare_to.size() ) !=0  ) {
+				break;
+			}
+			++i;
 		}
 	}
 
@@ -266,7 +306,7 @@ bool savegame_frame_t::check_file( const char *filename, const char *suffix )
  * This method is called if an action is triggered
  * @author Hj. Malthaner
  */
-bool savegame_frame_t::action_triggered( gui_action_creator_t *komp,value_t /* */)
+bool savegame_frame_t::action_triggered( gui_action_creator_t *komp, value_t /* */)
 {
 	char buf[1024];
 
@@ -274,13 +314,18 @@ bool savegame_frame_t::action_triggered( gui_action_creator_t *komp,value_t /* *
 		// Save/Load Button or Enter-Key pressed
 		//---------------------------------------
 
-		tstrncpy(buf, SAVE_PATH_X, lengthof(buf));
-		strcat(buf, ibuf);
-		strcat(buf, suffix);
-
+		if (strstr(ibuf,"net:")==ibuf) {
+			tstrncpy(buf,ibuf,lengthof(buf));
+		}
+		else {
+			tstrncpy(buf, SAVE_PATH_X, lengthof(buf));
+			strcat(buf, ibuf);
+			strcat(buf, suffix);
+		}
+		set_focus( NULL );
+		action(buf);
 		destroy_win(this);      //29-Oct-2001         Markus Weber    Close window
 
-		action(buf);
 	}
 	else if(komp == &cancelbutton) {
 		// Cancel-button pressed
@@ -291,8 +336,8 @@ bool savegame_frame_t::action_triggered( gui_action_creator_t *komp,value_t /* *
 	else {
 		// File in list selected
 		//--------------------------
-		for (slist_tpl<entry>::const_iterator i = entries.begin(), end = entries.end(); i != end  &&  !in_action; ++i) {
-			if (komp == i->button || komp == i->del) {
+		for(  slist_tpl<entry>::iterator i = entries.begin(), end = entries.end();  i != end  &&  !in_action;  ++i  ) {
+			if(  komp == i->button  ||  komp == i->del  ) {
 				in_action = true;
 				const bool action_btn = komp == i->button;
 				buf[0] = 0;
@@ -305,12 +350,29 @@ bool savegame_frame_t::action_triggered( gui_action_creator_t *komp,value_t /* *
 				}
 
 				if(action_btn) {
+					set_focus( NULL );
 					action(buf);
+					destroy_win(this);
 				}
 				else {
-					del_action(buf);
+					if(  del_action(buf)  ) {
+						set_focus( NULL );
+						destroy_win(this);
+					}
+					else {
+						set_focus(NULL);
+						// do not delete components
+						// simply hide them
+						i->button->set_visible(false);
+						i->del->set_visible(false);
+						i->label->set_visible(false);
+						i->button->set_groesse( koord( 0, 0 ) );
+						i->del->set_groesse( koord( 0, 0 ) );
+
+						resize( koord(0,0) );
+						in_action = false;
+					}
 				}
-				destroy_win(this);
 				break;
 			}
 		}
@@ -333,18 +395,27 @@ void savegame_frame_t::set_fenstergroesse(koord groesse)
 		groesse.y = display_get_height()-64;
 		// position adjustment will be done automatically ... nice!
 	}
-	scrolly.set_groesse( koord(groesse.x,groesse.y-30-40-8) );
 	gui_frame_t::set_fenstergroesse(groesse);
 	input.set_groesse(koord(groesse.x-75-10-10, 14));
+
+	sint16 y = 0;
 	for (slist_tpl<entry>::const_iterator i = entries.begin(), end = entries.end(); i != end; ++i) {
 		// resize all but delete button
-		// button_t*    button1 = i->del;
-		button_t*    button2 = i->button;
-		gui_label_t* label   = i->label;
-		button2->set_groesse(koord( groesse.x/2-40, 14));
-		label->set_pos(koord(groesse.x/2-40+30, label->get_pos().y));
+		if(  i->button->is_visible()  ) {
+			button_t*    button1 = i->del;
+			button1->set_pos( koord( button1->get_pos().x, y ) );
+			button_t*    button2 = i->button;
+			gui_label_t* label   = i->label;
+			button2->set_pos( koord( button2->get_pos().x, y ) );
+			button2->set_groesse(koord( groesse.x/2-40, 14));
+			label->set_pos(koord(groesse.x/2-40+30, y));
+			y += 14;
+		}
 	}
 
+	button_frame.set_groesse( koord( groesse.x, y ) );
+	scrolly.set_groesse( koord(groesse.x,groesse.y-30-40-8) );
+
 	divider1.set_pos(koord(10,groesse.y-44));
 	divider1.set_groesse(koord(groesse.x-20,0));
 	savebutton.set_pos(koord(10,groesse.y-34));
@@ -354,12 +425,16 @@ void savegame_frame_t::set_fenstergroesse(koord groesse)
 
 
 
-void savegame_frame_t::infowin_event(const event_t *ev)
+bool savegame_frame_t::infowin_event(const event_t *ev)
 {
-	if(ev->ev_class == INFOWIN && ev->ev_code == WIN_OPEN  &&  entries.empty()) {
+	if(ev->ev_class == INFOWIN  &&  ev->ev_code == WIN_OPEN  &&  entries.empty()) {
 		// before no virtual functions can be used ...
 		fill_list();
 		set_focus( &input );
 	}
-	gui_frame_t::infowin_event(ev);
+	if(  ev->ev_class == EVENT_KEYBOARD  &&  ev->ev_code == 13  ) {
+		action_triggered( &input, (long)0 );
+		return true;	// swallowed
+	}
+	return gui_frame_t::infowin_event(ev);
 }
