@@ -87,6 +87,7 @@ void ai_goods_t::rotate90( const sint16 y_size )
 	platz2.rotate90( y_size );
 	size1.rotate90( 0 );
 	size2.rotate90( 0 );
+	harbour.rotate90( y_size );
 }
 
 
@@ -981,13 +982,14 @@ DBG_MESSAGE("ai_goods_t::do_ki()","No roadway possible.");
 				while(  start_ware<ausgang.get_count()  &&  ausgang[start_ware].get_typ()!=freight  ) {
 					start_ware++;
 				}
-				koord harbour=platz1;
 				const int prod = min( ziel->get_base_production(), (start->get_base_production() * start->get_besch()->get_produkt(start_ware)->get_faktor()) - start->get_abgabe_letzt(start_ware) );
 				if(prod<0) {
 					// too much supplied last time?!? => retry
 					state = CHECK_CONVOI;
 					break;
 				}
+				// just remember the position, where the harbour will be built
+				harbour=platz1;
 				int ships_needed = 1 + (prod*koord_distance(harbour,start->get_pos().get_2d())) / (ship_vehicle->get_zuladung()*max(20,ship_vehicle->get_geschw()));
 				if(create_ship_transport_vehikel(start,ships_needed)) {
 					if(welt->lookup(harbour)->get_halt()->get_fab_list().is_contained(ziel)) {
@@ -1003,7 +1005,7 @@ DBG_MESSAGE("ai_goods_t::do_ki()","No roadway possible.");
 				}
 				else {
 					ship_vehicle = NULL;
-					state = CHECK_CONVOI;
+					state = NR_BAUE_CLEAN_UP;
 				}
 			}
 			break;
@@ -1086,33 +1088,30 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 			}
 			if(ship_vehicle) {
 				// only here, if we could built ships but no connection
-				halthandle_t start_halt;
-				for( int r=0;  r<4;  r++  ) {
-					start_halt = haltestelle_t::get_halt(welt,platz1+koord::nsow[r],this);
-					if(start_halt.is_bound()  &&  (start_halt->get_station_type()&haltestelle_t::dock)!=0) {
-						// delete all ships on this line
-						vector_tpl<linehandle_t> lines;
-						simlinemgmt.get_lines( simline_t::shipline, &lines );
-						if(!lines.empty()) {
-							linehandle_t line = lines.back();
-							schedule_t *fpl=line->get_schedule();
-							if(fpl->get_count()>1  &&  haltestelle_t::get_halt(welt,fpl->eintrag[0].pos,this)==start_halt) {
-								while(line->count_convoys()>0) {
-									convoihandle_t cnv = line->get_convoy(0);
-									cnv->self_destruct();
-									if(cnv.is_bound()) {
-										cnv->step();
-									}
+				halthandle_t start_halt = haltestelle_t::get_halt(welt,harbour,this);
+				if(start_halt.is_bound()  &&  (start_halt->get_station_type()&haltestelle_t::dock)!=0) {
+					// delete all ships on this line
+					vector_tpl<linehandle_t> lines;
+					simlinemgmt.get_lines( simline_t::shipline, &lines );
+					if(!lines.empty()) {
+						linehandle_t line = lines.back();
+						schedule_t *fpl=line->get_schedule();
+						if(fpl->get_count()>1  &&  haltestelle_t::get_halt(welt,fpl->eintrag[0].pos,this)==start_halt) {
+							while(line->count_convoys()>0) {
+								convoihandle_t cnv = line->get_convoy(0);
+								cnv->self_destruct();
+								if(cnv.is_bound()) {
+									cnv->step();
 								}
-								simlinemgmt.delete_line( line );
 							}
+							simlinemgmt.delete_line( line );
 						}
-						// delete harbour
-						call_general_tool( WKZ_REMOVER, platz1+koord::nsow[r], NULL );
-						break;
 					}
 				}
+				// delete harbour
+				call_general_tool( WKZ_REMOVER, harbour, NULL );
 			}
+			harbour = koord::invalid;
 			// otherwise it may always try to built the same route!
 			ziel = NULL;
 			// schilder aufraeumen
@@ -1132,6 +1131,7 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 			sprintf(buf, translator::translate("%s\nopened a new railway\nbetween %s\nat (%i,%i) and\n%s at (%i,%i)."), get_name(), translator::translate(start->get_name()), spos.x, spos.y, translator::translate(ziel->get_name()), zpos.x, zpos.y);
 			welt->get_message()->add_message(buf, spos.get_2d(), message_t::ai, PLAYER_FLAG|player_nr, rail_engine->get_basis_bild());
 
+			harbour = koord::invalid;
 			state = CHECK_CONVOI;
 		}
 		break;
@@ -1145,6 +1145,8 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 			const koord3d& zpos = ziel->get_pos();
 			sprintf(buf, translator::translate("%s\nnow operates\n%i trucks between\n%s at (%i,%i)\nand %s at (%i,%i)."), get_name(), count_road, translator::translate(start->get_name()), spos.x, spos.y, translator::translate(ziel->get_name()), zpos.x, zpos.y);
 			welt->get_message()->add_message(buf, spos.get_2d(), message_t::ai, PLAYER_FLAG|player_nr, road_vehicle->get_basis_bild());
+
+			harbour = koord::invalid;
 			state = CHECK_CONVOI;
 		}
 		break;
@@ -1282,7 +1284,7 @@ DBG_MESSAGE("ai_goods_t::step()","remove already constructed rail between %i,%i
 void ai_goods_t::rdwr(loadsave_t *file)
 {
 	if(  file->get_version()<102002  ) {
-		// do to an error the player was never saved correctly
+		// due to an error the player was never saved correctly
 		spieler_t::rdwr(file);
 		return;
 	}
@@ -1409,6 +1411,10 @@ void ai_goods_t::rdwr(loadsave_t *file)
 			}
 		}
 	}
+	// save harbour position
+	if (file->get_version() > 110000) {
+		harbour.rdwr(file);
+	}
 }
 
 bool ai_goods_t::is_forbidden( fabrik_t *fab1, fabrik_t *fab2, const ware_besch_t *w ) const
