@@ -8,8 +8,12 @@
 /* simple passenger AI (not using trains, not preoptimized network) */
 
 #include "../simcity.h"
+#include "../simfab.h"
 #include "../simhalt.h"
+#include "../simline.h"
+#include "../simmenu.h"
 #include "../simtools.h"
+#include "../simmesg.h"
 #include "../simworld.h"
 
 #include "../bauer/brueckenbauer.h"
@@ -20,6 +24,7 @@
 
 #include "../dataobj/loadsave.h"
 
+#include "../utils/cbuffer_t.h"
 #include "../utils/simstring.h"
 
 #include "../vehicle/simvehikel.h"
@@ -35,7 +40,7 @@ ai_passenger_t::ai_passenger_t(karte_t *wl, uint8 nr) : ai_t( wl, nr )
 	road_weg = NULL;
 
 	construction_speed = 8000;
-	next_contruction_steps = welt->get_steps()+simrand(construction_speed);
+	next_contruction_steps = welt->get_steps() + 50;
 
 	air_transport = true;
 	ship_transport = false;
@@ -100,18 +105,18 @@ koord ai_passenger_t::find_area_for_hub( const koord lo, const koord ru, const k
 				// flat, solid
 				if(  gr->get_typ()==grund_t::boden  &&  gr->get_grund_hang()==hang_t::flach  ) {
 					const ding_t* thing = gr->obj_bei(0);
-					int test_dist = abs_distance( trypos, basis );
-					if (thing == NULL || thing->get_besitzer() == NULL || thing->get_besitzer() == this) {
-						if(gr->is_halt()  &&  check_owner( this, gr->get_halt()->get_besitzer())  &&  gr->hat_weg(road_wt)) {
+					int test_dist = koord_distance( trypos, basis );
+					if (!thing || !thing->get_besitzer() || thing->get_besitzer() == sim::up_cast<spieler_t const*>(this)) {
+						if(  gr->is_halt()  &&  check_owner( gr->get_halt()->get_besitzer(), this )  &&  gr->hat_weg(road_wt)  ) {
 							// ok, one halt belongs already to us ... (should not really happen!) but might be a public stop
 							return trypos;
-						} else if(test_dist<dist  &&  gr->hat_weg(road_wt)  &&  !gr->is_halt()  ) {
+						} else if(  test_dist<dist  &&  gr->hat_weg(road_wt)  &&  !gr->is_halt()  ) {
 							ribi_t::ribi  ribi = gr->get_weg_ribi_unmasked(road_wt);
 							if(  ribi_t::ist_gerade(ribi)  ||  ribi_t::ist_einfach(ribi)  ) {
 								best_pos = trypos;
 								dist = test_dist;
 							}
-						} else if(test_dist+2<dist  &&  gr->ist_natur()  ) {
+						} else if(  test_dist+2<dist  &&  gr->ist_natur()  ) {
 							// also ok for a stop, but second choice
 							// so wie gave it a malus of 2
 							best_pos = trypos;
@@ -150,7 +155,7 @@ koord ai_passenger_t::find_harbour_pos(karte_t* welt, const stadt_t *s )
 	// try to find an airport place as close to the city as possible
 	for(  k.y=max(6,s->get_linksoben().y-10); k.y<=min(welt->get_groesse_y()-3-6,s->get_rechtsunten().y+10); k.y++  ) {
 		for(  k.x=max(6,s->get_linksoben().x-25); k.x<=min(welt->get_groesse_x()-3-6,s->get_rechtsunten().x+10); k.x++  ) {
-			sint32 testdist = abs_distance( k, s->get_pos() );
+			sint32 testdist = koord_distance( k, s->get_pos() );
 			if(  testdist<bestdist  ) {
 				if(  k.x+2<s->get_linksoben().x  ||  k.y+2<s->get_linksoben().y  ||  k.x>=s->get_rechtsunten().x  ||  k.y>=s->get_rechtsunten().y  ) {
 					// malus for out of town
@@ -185,6 +190,11 @@ bool ai_passenger_t::create_water_transport_vehikel(const stadt_t* start_stadt,
 		return false;
 	}
 
+	if(  convoihandle_t::is_exhausted()  ) {
+		// too many convois => cannot do anything about this ...
+		return false;
+	}
+
 	stadt_t *end_stadt = NULL;
 	grund_t *ziel = welt->lookup_kartenboden(target_pos);
 	gebaeude_t *gb = ziel->find<gebaeude_t>();
@@ -262,6 +272,7 @@ bool ai_passenger_t::create_water_transport_vehikel(const stadt_t* start_stadt,
 		// we use the free own vehikel_besch_t
 		vehikel_besch_t remover_besch( water_wt, 500, vehikel_besch_t::diesel );
 		vehikel_t* test_driver = vehikelbauer_t::baue( koord3d(start_harbour-start_dx,welt->get_grundwasser()), this, NULL, &remover_besch );
+		test_driver->set_flag( ding_t::not_on_map );
 		route_t verbindung;
 		bool connected = verbindung.calc_route(welt, koord3d(start_harbour-start_dx,welt->get_grundwasser()), koord3d(end_harbour-end_dx,welt->get_grundwasser()), test_driver, 0);
 		delete test_driver;
@@ -283,7 +294,7 @@ bool ai_passenger_t::create_water_transport_vehikel(const stadt_t* start_stadt,
 			bauigel.set_keep_city_roads(true);
 			bauigel.set_maximum(10000);
 			bauigel.calc_route( welt->lookup_kartenboden(bushalt)->get_pos(), welt->lookup_kartenboden(town_road)->get_pos() );
-			if(bauigel.max_n <= 1) {
+			if(bauigel.get_count()-1 <= 1) {
 				return false;
 			}
 			bauigel.baue();
@@ -301,7 +312,7 @@ bool ai_passenger_t::create_water_transport_vehikel(const stadt_t* start_stadt,
 			bauigel.set_keep_city_roads(true);
 			bauigel.set_maximum(10000);
 			bauigel.calc_route( welt->lookup_kartenboden(bushalt)->get_pos(), welt->lookup_kartenboden(town_road)->get_pos() );
-			if(bauigel.max_n <= 1) {
+			if(bauigel.get_count()-1 <= 1) {
 				return false;
 			}
 			bauigel.baue();
@@ -320,7 +331,7 @@ bool ai_passenger_t::create_water_transport_vehikel(const stadt_t* start_stadt,
 		}
 		// and change name to dock ...
 		halthandle_t halt = welt->lookup(bushalt)->get_halt();
-		char *name = halt->create_name(bushalt, "Dock");
+		char *name = halt->create_name(bushalt, "Dock", welt->get_einstellungen()->get_name_language_id() );
 		halt->set_name( name );
 		free(name);
 		// finally built the dock
@@ -349,7 +360,7 @@ bool ai_passenger_t::create_water_transport_vehikel(const stadt_t* start_stadt,
 		}
 		// and change name to dock ...
 		halthandle_t halt = welt->lookup(bushalt)->get_halt();
-		char *name = halt->create_name(bushalt, "Dock");
+		char *name = halt->create_name(bushalt, "Dock", welt->get_einstellungen()->get_name_language_id() );
 		halt->set_name( name );
 		free(name);
 		// finally built the dock
@@ -377,7 +388,7 @@ bool ai_passenger_t::create_water_transport_vehikel(const stadt_t* start_stadt,
 			if(plan) {
 				grund_t *gr = plan->get_kartenboden();
 				if(  gr->ist_wasser()  &&  !gr->get_halt().is_bound()  ) {
-					if(plan->get_haltlist_count()>=1  &&  plan->get_haltlist()[0]==start_hub  &&  abs_distance(start_pos,end_harbour)>abs_distance(p,end_harbour)) {
+					if(plan->get_haltlist_count()>=1  &&  plan->get_haltlist()[0]==start_hub  &&  koord_distance(start_pos,end_harbour)>koord_distance(p,end_harbour)) {
 						start_pos = p;
 					}
 				}
@@ -394,7 +405,7 @@ bool ai_passenger_t::create_water_transport_vehikel(const stadt_t* start_stadt,
 			if(plan) {
 				grund_t *gr = plan->get_kartenboden();
 				if(  gr->ist_wasser()  &&  !gr->get_halt().is_bound()  ) {
-					if(plan->get_haltlist_count()>=1  &&  plan->get_haltlist()[0]==end_hub  &&  abs_distance(end_pos,start_harbour)>abs_distance(p,start_harbour)) {
+					if(plan->get_haltlist_count()>=1  &&  plan->get_haltlist()[0]==end_hub  &&  koord_distance(end_pos,start_harbour)>koord_distance(p,start_harbour)) {
 						end_pos = p;
 					}
 				}
@@ -407,10 +418,11 @@ bool ai_passenger_t::create_water_transport_vehikel(const stadt_t* start_stadt,
 	fpl->append( welt->lookup_kartenboden(start_pos), 0, 0 );
 	fpl->append( welt->lookup_kartenboden(end_pos), 90, 0 );
 	fpl->set_aktuell( 1 );
+	fpl->eingabe_abschliessen();
 	linehandle_t line=simlinemgmt.create_line(simline_t::shipline,this,fpl);
 	delete fpl;
 
-	// now create one plane
+	// now create one ship
 	vehikel_t* v = vehikelbauer_t::baue( koord3d(start_pos,welt->get_grundwasser()), this, NULL, v_besch);
 	convoi_t* cnv = new convoi_t(this);
 	cnv->set_name(v->get_besch()->get_name());
@@ -486,7 +498,7 @@ halthandle_t ai_passenger_t::build_airport(const stadt_t* city, koord pos, int r
 		return halthandle_t();
 	}
 	// ok, now we could built it => flatten the land
-	sint16 h = max( welt->get_grundwasser()+Z_TILE_STEP, welt->lookup_kartenboden(pos)->get_hoehe() );
+	sint8 h = max( welt->get_grundwasser()+Z_TILE_STEP, welt->lookup_kartenboden(pos)->get_hoehe() );
 	const koord dx( size.x/2, size.y/2 );
 	for(  sint16 i=0;  i!=size.y+dx.y;  i+=dx.y  ) {
 		for( sint16 j=0;  j!=size.x+dx.x;  j+=dx.x  ) {
@@ -501,16 +513,16 @@ halthandle_t ai_passenger_t::build_airport(const stadt_t* city, koord pos, int r
 	koord center=pos+dx;
 	bauigel.route_fuer( wegbauer_t::luft, taxi_besch, NULL, NULL );
 	bauigel.calc_straight_route( welt->lookup_kartenboden(center+koord::nord)->get_pos(), welt->lookup_kartenboden(center+koord::sued)->get_pos() );
-	assert(bauigel.max_n > 1);
+	assert(bauigel.get_count()-1 > 1);
 	bauigel.baue();
 	bauigel.route_fuer( wegbauer_t::luft, taxi_besch, NULL, NULL );
 	bauigel.calc_straight_route( welt->lookup_kartenboden(center+koord::west)->get_pos(), welt->lookup_kartenboden(center+koord::ost)->get_pos() );
-	assert(bauigel.max_n > 1);
+	assert(bauigel.get_count()-1 > 1);
 	bauigel.baue();
 	// now try to connect one of the corners with a road
 	koord bushalt = koord::invalid, runwaystart, runwayend;
 	koord trypos[4] = { koord(0,0), koord(size.x,0), koord(0,size.y), koord(size.x,size.y) };
-	sint32 lenght=9999;
+	uint32 length=9999;
 	rotation=-1;
 
 	bauigel.route_fuer( wegbauer_t::strasse, wegbauer_t::weg_search( road_wt, 25, welt->get_timeline_year_month(), weg_t::type_flat ), tunnelbauer_t::find_tunnel(road_wt,road_vehicle->get_geschw(),welt->get_timeline_year_month()), brueckenbauer_t::find_bridge(road_wt,road_vehicle->get_geschw(),welt->get_timeline_year_month()) );
@@ -523,9 +535,9 @@ halthandle_t ai_passenger_t::build_airport(const stadt_t* city, koord pos, int r
 		bushalt = pos+trypos[i];
 		bauigel.calc_route(welt->lookup_kartenboden(bushalt)->get_pos(),welt->lookup_kartenboden(town_road)->get_pos());
 		// no road => try next
-		if(  bauigel.max_n>=1  &&   bauigel.max_n<lenght  ) {
+		if(  bauigel.get_count() >= 2  &&   bauigel.get_count() < length+1  ) {
 			rotation = i;
-			lenght = bauigel.max_n;
+			length = bauigel.get_count()-1;
 		}
 	}
 
@@ -556,24 +568,24 @@ halthandle_t ai_passenger_t::build_airport(const stadt_t* city, koord pos, int r
 	}
 	// and change name to airport ...
 	halthandle_t halt = welt->lookup(bushalt)->get_halt();
-	char *name = halt->create_name( bushalt, "Airport" );
+	char *name = halt->create_name( bushalt, "Airport", welt->get_einstellungen()->get_name_language_id() );
 	halt->set_name( name );
 	free(name);
 	// built also runway now ...
 	bauigel.route_fuer( wegbauer_t::luft, runway_besch, NULL, NULL );
 	bauigel.calc_straight_route( welt->lookup_kartenboden(pos+trypos[rotation==0?3:0])->get_pos(), welt->lookup_kartenboden(pos+trypos[1+(rotation&1)])->get_pos() );
-	assert(bauigel.max_n > 1);
+	assert(bauigel.get_count()-1 > 1);
 	bauigel.baue();
 	// now the airstops (only on single tiles, this will always work
 	const haus_besch_t* airstop_besch = hausbauer_t::get_random_station(haus_besch_t::generic_stop, air_wt, welt->get_timeline_year_month(), 0 );
 	for(  int i=0;  i<4;  i++  ) {
-		if(  abs_distance(center+koord::nsow[i],bushalt)==1  &&  ribi_t::ist_einfach( welt->lookup_kartenboden(center+koord::nsow[i])->get_weg_ribi_unmasked(air_wt) )  ) {
+		if(  koord_distance(center+koord::nsow[i],bushalt)==1  &&  ribi_t::ist_einfach( welt->lookup_kartenboden(center+koord::nsow[i])->get_weg_ribi_unmasked(air_wt) )  ) {
 			call_general_tool( WKZ_STATION, center+koord::nsow[i], airstop_besch->get_name() );
 		}
 	}
 	// and now the one far away ...
 	for(  int i=0;  i<4;  i++  ) {
-		if(  abs_distance(center+koord::nsow[i],bushalt)>1  &&  ribi_t::ist_einfach( welt->lookup_kartenboden(center+koord::nsow[i])->get_weg_ribi_unmasked(air_wt) )  ) {
+		if(  koord_distance(center+koord::nsow[i],bushalt)>1  &&  ribi_t::ist_einfach( welt->lookup_kartenboden(center+koord::nsow[i])->get_weg_ribi_unmasked(air_wt) )  ) {
 			call_general_tool( WKZ_STATION, center+koord::nsow[i], airstop_besch->get_name() );
 		}
 	}
@@ -591,7 +603,7 @@ static koord find_airport_pos(karte_t* welt, const stadt_t *s )
 	// try to find an airport place as close to the city as possible
 	for(  k.y=max(6,s->get_linksoben().y-10); k.y<=min(welt->get_groesse_y()-3-6,s->get_rechtsunten().y+10); k.y++  ) {
 		for(  k.x=max(6,s->get_linksoben().x-25); k.x<=min(welt->get_groesse_x()-3-6,s->get_rechtsunten().x+10); k.x++  ) {
-			sint32 testdist = abs_distance( k, s->get_pos() );
+			sint32 testdist = koord_distance( k, s->get_pos() );
 			if(  testdist<bestdist  ) {
 				if(  k.x+2<s->get_linksoben().x  ||  k.y+2<s->get_linksoben().y  ||  k.x>=s->get_rechtsunten().x  ||  k.y>=s->get_rechtsunten().y  ) {
 					// malus for out of town
@@ -620,6 +632,11 @@ bool ai_passenger_t::create_air_transport_vehikel(const stadt_t *start_stadt, co
 		return false;
 	}
 
+	if(  convoihandle_t::is_exhausted()  ) {
+		// too many convois => cannot do anything about this ...
+		return false;
+	}
+
 	halthandle_t start_hub = get_our_hub( start_stadt );
 	halthandle_t start_connect_hub = halthandle_t();
 	koord start_airport;
@@ -731,6 +748,7 @@ bool ai_passenger_t::create_air_transport_vehikel(const stadt_t *start_stadt, co
 	fpl->append( start, 0, 0 );
 	fpl->append( end, 90, 0 );
 	fpl->set_aktuell( 1 );
+	fpl->eingabe_abschliessen();
 	linehandle_t line=simlinemgmt.create_line(simline_t::airline,this,fpl);
 	delete fpl;
 
@@ -767,21 +785,26 @@ void ai_passenger_t::create_bus_transport_vehikel(koord startpos2d,int anz_vehik
 {
 DBG_MESSAGE("ai_passenger_t::create_bus_transport_vehikel()","bus at (%i,%i)",startpos2d.x,startpos2d.y);
 	// now start all vehicle one field before, so they load immediately
-	koord3d startpos = welt->lookup(startpos2d)->get_kartenboden()->get_pos();
+	koord3d startpos = welt->lookup_kartenboden(startpos2d)->get_pos();
 
 	// since 86.01 we use lines for road vehicles ...
 	schedule_t *fpl=new autofahrplan_t();
 	// do not start at current stop => wont work ...
 	for(int j=0;  j<anzahl;  j++) {
-		fpl->append(welt->lookup(stops[j])->get_kartenboden(), j == 0 || !do_wait ? 0 : 10);
+		fpl->append(welt->lookup_kartenboden(stops[j]), j == 0 || !do_wait ? 0 : 10);
 	}
 	fpl->set_aktuell( stops[0]==startpos2d );
+	fpl->eingabe_abschliessen();
 	linehandle_t line=simlinemgmt.create_line(simline_t::truckline,this,fpl);
 	delete fpl;
 
 	// now create all vehicles as convois
 	for(int i=0;  i<anz_vehikel;  i++) {
 		vehikel_t* v = vehikelbauer_t::baue(startpos, this, NULL, road_vehicle);
+		if(  convoihandle_t::is_exhausted()  ) {
+			// too many convois => cannot do anything about this ...
+			return;
+		}
 		convoi_t* cnv = new convoi_t(this);
 		// V.Meyer: give the new convoi name from first vehicle
 		cnv->set_name(v->get_besch()->get_name());
@@ -796,8 +819,7 @@ DBG_MESSAGE("ai_passenger_t::create_bus_transport_vehikel()","bus at (%i,%i)",st
 
 // now we follow all adjacent streets recursively and mark them
 // if they below to this stop, then we continue
-void
-ai_passenger_t::walk_city( linehandle_t &line, grund_t *&start, const int limit )
+void ai_passenger_t::walk_city( linehandle_t &line, grund_t *&start, const int limit )
 {
 	//maximum number of stops reached?
 	if(line->get_schedule()->get_count()>=limit)  {
@@ -815,7 +837,7 @@ ai_passenger_t::walk_city( linehandle_t &line, grund_t *&start, const int limit
 
 		// ok, if connected, not marked, and not owner by somebody else
 		grund_t *to;
-		if(start->get_neighbour(to, road_wt, koord::nsow[r] )  &&  !welt->ist_markiert(to)  &&  check_owner(this, to->obj_bei(0)->get_besitzer())) {
+		if(  start->get_neighbour(to, road_wt, koord::nsow[r] )  &&  !welt->ist_markiert(to)  &&  check_owner(to->obj_bei(0)->get_besitzer(),this)  ) {
 
 			// ok, here is a valid street tile
 			welt->markiere(to);
@@ -882,6 +904,11 @@ ai_passenger_t::walk_city( linehandle_t &line, grund_t *&start, const int limit
  */
 void ai_passenger_t::cover_city_with_bus_route(koord start_pos, int number_of_stops)
 {
+	if(  convoihandle_t::is_exhausted()  ) {
+		// too many convois => cannot do anything about this ...
+		return;
+	}
+
 	// nothing in lists
 	welt->unmarkiere_alle();
 
@@ -892,6 +919,7 @@ void ai_passenger_t::cover_city_with_bus_route(koord start_pos, int number_of_st
 
 	// now create a line
 	walk_city( line, start, number_of_stops );
+	line->get_schedule()->eingabe_abschliessen();
 
 	road_vehicle = vehikelbauer_t::vehikel_search( road_wt, welt->get_timeline_year_month(), 1, 50, warenbauer_t::passagiere, false, false );
 	if( line->get_schedule()->get_count()>1  ) {
@@ -941,7 +969,7 @@ void ai_passenger_t::step()
 			 * The second condition may happen due to extensive replacement operations;
 			 * in such a case it is save enough to expand anyway.
 			 */
-			if(!(konto>0  ||  finance_history_month[0][COST_ASSETS]+konto>welt->get_einstellungen()->get_starting_money())  ) {
+			if(!(konto>0  ||  finance_history_month[0][COST_ASSETS]+konto>starting_money)  ) {
 				return;
 			}
 
@@ -1011,9 +1039,9 @@ DBG_MESSAGE("ai_passenger_t::do_passenger_ki()","searching attraction");
 						// in any case we do not want to serve this location already
 						const koord cov = koord(welt->get_einstellungen()->get_station_coverage(),welt->get_einstellungen()->get_station_coverage());
 						koord test_platz=find_area_for_hub(pos-cov,pos+size+cov,pos);
-						if(!is_my_halt(test_platz)) {
+						if(  !haltestelle_t::get_halt(welt,test_platz,this).is_bound()  ) {
 							// not served
-							dist = abs_distance(platz1,test_platz);
+							dist = koord_distance(platz1,test_platz);
 							if(dist+simrand(50)<last_dist  &&   dist>3) {
 								// but closer than the others
 								if(ausflug) {
@@ -1044,7 +1072,7 @@ DBG_MESSAGE("ai_passenger_t::do_passenger_ki()","searching town");
 					const stadt_t* cur = staedte[nr];
 					if(cur!=last_start_stadt  &&  cur!=start_stadt) {
 						halthandle_t end_halt = get_our_hub(cur);
-						int dist = abs_distance(platz1,cur->get_pos());
+						int dist = koord_distance(platz1,cur->get_pos());
 						if(  end_halt.is_bound()  &&  is_connected(platz1,end_halt->get_basis_pos(),warenbauer_t::passagiere) ) {
 							// already connected
 							continue;
@@ -1133,14 +1161,14 @@ DBG_MESSAGE("ai_passenger_t::do_passenger_ki()","using %s on %s",road_vehicle->g
 			if(bs  &&  create_simple_road_transport(platz1, koord(1,1),platz2,koord(1,1),road_weg)  ) {
 				// since the road my have led to a crossing at the indended stop position ...
 				bool ok = true;
-				if(!is_my_halt(platz1)) {
+				if(  !haltestelle_t::get_halt(welt,platz1,this).is_bound()  ) {
 					if(  !call_general_tool( WKZ_STATION, platz1, bs->get_name() )  ) {
 						platz1 = find_area_for_hub( platz1-koord(2,2), platz1+koord(2,2), platz1 );
 						ok = call_general_tool( WKZ_STATION, platz1, bs->get_name() );
 					}
 				}
 				if(  ok  ) {
-					if(!is_my_halt(platz2)) {
+					if(  !haltestelle_t::get_halt(welt,platz2,this).is_bound()  ) {
 						if(  !call_general_tool( WKZ_STATION, platz2, bs->get_name() )  ) {
 							platz2 = find_area_for_hub( platz2-koord(2,2), platz2+koord(2,2), platz2 );
 							ok = call_general_tool( WKZ_STATION, platz2, bs->get_name() );
@@ -1171,7 +1199,7 @@ DBG_MESSAGE("ai_passenger_t::do_passenger_ki()","using %s on %s",road_vehicle->g
 						cover_city_with_bus_route(platz1, 6);
 						cover_city_with_bus_route(platz2, 6);
 					}
-					welt->get_message()->add_message((const char *)buf,platz1,message_t::ai,PLAYER_FLAG|player_nr,road_vehicle->get_basis_bild());
+					welt->get_message()->add_message(buf, platz1, message_t::ai, PLAYER_FLAG|player_nr, road_vehicle->get_basis_bild());
 				}
 			}
 		}
@@ -1191,7 +1219,7 @@ DBG_MESSAGE("ai_passenger_t::do_passenger_ki()","using %s on %s",road_vehicle->g
 				}
 				cbuffer_t buf(1024);
 				buf.printf( translator::translate("Ferry service by\n%s\nnow between\n%s \nand %s.\n"), get_name(), start_stadt->get_name(), end_stadt->get_name() );
-				welt->get_message()->add_message((const char *)buf,end_stadt->get_pos(),message_t::ai,PLAYER_FLAG|player_nr,road_vehicle->get_basis_bild());
+				welt->get_message()->add_message(buf, end_stadt->get_pos(), message_t::ai, PLAYER_FLAG | player_nr, road_vehicle->get_basis_bild());
 				state = NR_SUCCESS;
 			}
 			else {
@@ -1207,7 +1235,7 @@ DBG_MESSAGE("ai_passenger_t::do_passenger_ki()","using %s on %s",road_vehicle->g
 		// despite its name: try airplane
 		case NR_BAUE_AIRPORT_ROUTE:
 			// try airline (if we are wealthy enough) ...
-			if(  !air_transport  ||  finance_history_month[1][COST_CASH]<welt->get_einstellungen()->get_starting_money()  ||  !create_air_transport_vehikel( start_stadt, end_stadt )) {
+			if(  !air_transport  ||  finance_history_month[1][COST_CASH]<starting_money  ||  !create_air_transport_vehikel( start_stadt, end_stadt )) {
 				state = NR_BAUE_CLEAN_UP;
 			}
 			else {
@@ -1216,7 +1244,7 @@ DBG_MESSAGE("ai_passenger_t::do_passenger_ki()","using %s on %s",road_vehicle->g
 				cover_city_with_bus_route( get_our_hub(end_stadt)->get_basis_pos(), 6);
 				cbuffer_t buf(1024);
 				buf.printf( translator::translate("Airline service by\n%s\nnow between\n%s \nand %s.\n"), get_name(), start_stadt->get_name(), end_stadt->get_name() );
-				welt->get_message()->add_message((const char *)buf,end_stadt->get_pos(),message_t::ai,PLAYER_FLAG|player_nr,road_vehicle->get_basis_bild());
+				welt->get_message()->add_message(buf, end_stadt->get_pos(), message_t::ai, PLAYER_FLAG | player_nr, road_vehicle->get_basis_bild());
 				state = NR_SUCCESS;
 			}
 		break;
@@ -1259,7 +1287,7 @@ DBG_MESSAGE("ai_passenger_t::do_passenger_ki()","using %s on %s",road_vehicle->g
 				}
 
 				// avoid empty schedule ?!?
-				assert(line->get_schedule()->get_count()>0);
+				assert(!line->get_schedule()->empty());
 
 				// made loss with this line
 				if(line->get_finance_history(0,LINE_PROFIT)<0) {
@@ -1272,15 +1300,21 @@ DBG_MESSAGE("ai_passenger_t::do_passenger_ki()","using %s on %s",road_vehicle->g
 							convoihandle_t cnv = line->get_convoy(i);
 							if(cnv->has_obsolete_vehicles()) {
 								obsolete.append(cnv);
-								capacity += cnv->get_vehikel(0)->get_besch()->get_zuladung();
+								capacity += cnv->front()->get_besch()->get_zuladung();
 							}
 						}
 						if(capacity>0) {
 							// now try to finde new vehicle
-							const vehikel_besch_t *v_besch = vehikelbauer_t::vehikel_search( line->get_convoy(0)->get_vehikel(0)->get_waytype(), welt->get_current_month(), 50, welt->get_average_speed(line->get_convoy(0)->get_vehikel(0)->get_waytype()), warenbauer_t::passagiere, false, false );
-							if(  !v_besch->is_retired(welt->get_current_month())  &&  v_besch!=line->get_convoy(0)->get_vehikel(0)->get_besch()) {
+							vehikel_t              const& v       = *line->get_convoy(0)->front();
+							waytype_t              const  wt      = v.get_waytype();
+							vehikel_besch_t const* const  v_besch = vehikelbauer_t::vehikel_search(wt, welt->get_current_month(), 50, welt->get_average_speed(wt), warenbauer_t::passagiere, false, false);
+							if (!v_besch->is_retired(welt->get_current_month()) && v_besch != v.get_besch()) {
 								// there is a newer one ...
 								for(  uint32 new_capacity=0;  capacity>new_capacity;  new_capacity+=v_besch->get_zuladung()) {
+									if(  convoihandle_t::is_exhausted()  ) {
+										// too many convois => cannot do anything about this ...
+										break;
+									}
 									vehikel_t* v = vehikelbauer_t::baue( line->get_schedule()->eintrag[0].pos, this, NULL, v_besch  );
 									convoi_t* new_cnv = new convoi_t(this);
 									new_cnv->set_name( v->get_besch()->get_name() );
@@ -1310,9 +1344,9 @@ DBG_MESSAGE("ai_passenger_t::do_passenger_ki()","using %s on %s",road_vehicle->g
 				sint32 ratio = (sint32)((free_cap*100l)/(free_cap+used_cap));
 
 				// next: check for overflowing lines, i.e. running with 3/4 of the capacity
-				if(  ratio<10  ) {
+				if(  ratio<10  &&  !convoihandle_t::is_exhausted()  ) {
 					// else add the first convoi again
-					vehikel_t* v = vehikelbauer_t::baue( line->get_schedule()->eintrag[0].pos, this, NULL, line->get_convoy(0)->get_vehikel(0)->get_besch()  );
+					vehikel_t* const v = vehikelbauer_t::baue(line->get_schedule()->eintrag[0].pos, this, NULL, line->get_convoy(0)->front()->get_besch());
 					convoi_t* new_cnv = new convoi_t(this);
 					new_cnv->set_name( v->get_besch()->get_name() );
 					new_cnv->add_vehikel( v );
@@ -1348,6 +1382,12 @@ DBG_MESSAGE("ai_passenger_t::do_passenger_ki()","using %s on %s",road_vehicle->g
 
 void ai_passenger_t::rdwr(loadsave_t *file)
 {
+	if(  file->get_version()<102002  ) {
+		// do to an error the player was never saved correctly
+		spieler_t::rdwr(file);
+		return;
+	}
+
 	xml_tag_t t( file, "ai_passenger_t" );
 
 	// first: do all the administration
@@ -1363,17 +1403,17 @@ void ai_passenger_t::rdwr(loadsave_t *file)
 	}
 
 	// now save current state ...
-	file->rdwr_enum(state, "");
-	file->rdwr_long( construction_speed, "" );
-	file->rdwr_bool( air_transport, "" );
-	file->rdwr_bool( ship_transport, "" );
+	file->rdwr_enum(state);
+	file->rdwr_long(construction_speed);
+	file->rdwr_bool(air_transport);
+	file->rdwr_bool(ship_transport);
 	platz1.rdwr( file );
 	platz2.rdwr( file );
 
 	if(file->is_saving()) {
 		// save current pointers
 		sint32 delta_steps = next_contruction_steps-welt->get_steps();
-		file->rdwr_long(delta_steps, " ");
+		file->rdwr_long(delta_steps);
 		koord k = start_stadt ? start_stadt->get_pos() : koord::invalid;
 		k.rdwr(file);
 		k = end_stadt ? end_stadt->get_pos() : koord::invalid;
@@ -1385,7 +1425,7 @@ void ai_passenger_t::rdwr(loadsave_t *file)
 	}
 	else {
 		// since steps in loaded game == 0
-		file->rdwr_long(next_contruction_steps, " ");
+		file->rdwr_long(next_contruction_steps);
 		next_contruction_steps += welt->get_steps();
 		// reinit current pointers
 		koord k;
@@ -1412,31 +1452,10 @@ void ai_passenger_t::rdwr(loadsave_t *file)
  */
 void ai_passenger_t::bescheid_vehikel_problem(convoihandle_t cnv,const koord3d ziel)
 {
-	switch(cnv->get_state()) {
-
-		case convoi_t::NO_ROUTE:
-DBG_MESSAGE("ai_passenger_t::bescheid_vehikel_problem","Vehicle %s can't find a route to (%i,%i)!", cnv->get_name(),ziel.x,ziel.y);
-			if(this==welt->get_active_player()) {
-				char buf[256];
-				sprintf(buf,translator::translate("Vehicle %s can't find a route!"), cnv->get_name());
-				welt->get_message()->add_message(buf, cnv->get_pos().get_2d(),message_t::convoi,PLAYER_FLAG|player_nr,cnv->get_vehikel(0)->get_basis_bild());
-			}
-			else {
-				cnv->self_destruct();
-			}
-			break;
-
-		case convoi_t::WAITING_FOR_CLEARANCE_ONE_MONTH:
-		case convoi_t::CAN_START_ONE_MONTH:
-DBG_MESSAGE("ai_passenger_t::bescheid_vehikel_problem","Vehicle %s stucked!", cnv->get_name(),ziel.x,ziel.y);
-			if(this==welt->get_active_player()) {
-				char buf[256];
-				sprintf(buf,translator::translate("Vehicle %s is stucked!"), cnv->get_name());
-				welt->get_message()->add_message(buf, cnv->get_pos().get_2d(),message_t::convoi,PLAYER_FLAG|player_nr,cnv->get_vehikel(0)->get_basis_bild());
-			}
-			break;
-
-		default:
-DBG_MESSAGE("ai_passenger_t::bescheid_vehikel_problem","Vehicle %s, state %i!", cnv->get_name(), cnv->get_state());
+	if(  cnv->get_state() == convoi_t::NO_ROUTE  &&  this!=welt->get_active_player()  ) {
+			DBG_MESSAGE("ai_passenger_t::bescheid_vehikel_problem","Vehicle %s can't find a route to (%i,%i)!", cnv->get_name(),ziel.x,ziel.y);
+			cnv->self_destruct();
+			return;
 	}
+	spieler_t::bescheid_vehikel_problem( cnv, ziel );
 }
