@@ -13,21 +13,20 @@
 
 #include "tpl/vector_tpl.h"
 #include "tpl/weighted_vector_tpl.h"
-#include "tpl/array2d_tpl.h"
+#include "tpl/sparse_tpl.h"
+
+#include <string>
 
 class karte_t;
 class spieler_t;
-class cbuffer_t;
-class cstring_t;
-
-// part of passengers going to factories or toursit attractions (100% mx)
-#define FACTORY_PAX (33)	// workers
-#define TOURIST_PAX (16)		// tourists
+class rule_t;
 
 
 #define MAX_CITY_HISTORY_YEARS  (12) // number of years to keep history
 #define MAX_CITY_HISTORY_MONTHS (12) // number of months to keep history
 
+#define PAX_DESTINATIONS_SIZE (128) // size of the minimap.
+
 enum city_cost {
 	HIST_CITICENS=0,// total people
 	HIST_GROWTH,	// growth (just for convenience)
@@ -78,10 +77,22 @@ class stadt_t
 
 public:
 	/**
-	 * Reads city configuration data
+	 * Reads city configuration data from config/cityrules.tab
 	 * @author Hj. Malthaner
 	 */
-	static bool cityrules_init(cstring_t objpathname);
+	static bool cityrules_init(const std::string &objpathname);
+	/**
+	 * Reads/writes city configuration data from/to a savegame
+	 * called from einstellungen_t::rdwr
+	 * only written for networkgames
+	 * @author Dwachs
+	 */
+	static void cityrules_rdwr(loadsave_t *file);
+
+	static uint32 get_industry_increase();
+	static void set_industry_increase(uint32 ind_increase);
+	static uint32 get_minimum_city_distance();
+	static void set_minimum_city_distance(uint32 s);
 
 private:
 	static karte_t *welt;
@@ -90,13 +101,19 @@ private:
 
 	weighted_vector_tpl <gebaeude_t *> buildings;
 
-	array2d_tpl<uint8> pax_ziele_alt;
-	array2d_tpl<uint8> pax_ziele_neu;
+	sparse_tpl<uint8> pax_destinations_old;
+	sparse_tpl<uint8> pax_destinations_new;
+
+	// this counter will increment by one for every change => dialogs can question, if they need to update map
+	unsigned long pax_destinations_new_change;
 
 	koord pos;			// Gruendungsplanquadrat der Stadt
+	koord townhall_road; // road in front of townhall
 	koord lo, ur;		// max size of housing area
 	bool  has_low_density;	// in this case extend borders by two
 
+	bool allow_citygrowth;	// town can be static and will grow (true by default)
+
 	// this counter indicate which building will be processed next
 	uint32 step_count;
 
@@ -176,13 +193,10 @@ private:
 	weighted_vector_tpl<fabrik_t *> arbeiterziele;
 
 	/**
-	 * allokiert pax_ziele_alt/neu und init. die werte
+	 * Initialization of pax_destinations_old/new
 	 * @author Hj. Malthaner
 	 */
-	void init_pax_ziele();
-
-	// recalculate house informations (used for target selection)
-	void recount_houses();
+	void init_pax_destinations();
 
 	// recalcs city borders (after loading and deletion)
 	void recalc_city_size();
@@ -259,16 +273,21 @@ private:
 	 * @return true on match, false otherwise
 	 * @author Hj. Malthaner
 	 */
-	bool bewerte_loc(koord pos, const char *regel, int rotation);
+	bool bewerte_loc(koord pos, rule_t &regel, int rotation);
+
+	/*
+	 * evaluates the location, tests again all rules, and caches the result
+	 */
+	uint16 bewerte_loc_cache(const koord pos, bool force=false);
 
 	/**
 	 * Check rule in all transformations at given position
 	 * @author Hj. Malthaner
 	 */
-	sint32 bewerte_pos(koord pos, const char *regel);
+	sint32 bewerte_pos(koord pos, rule_t &regel);
 
-	void bewerte_strasse(koord pos, sint32 rd, const char* regel);
-	void bewerte_haus(koord pos, sint32 rd, const char* regel);
+	void bewerte_strasse(koord pos, sint32 rd, rule_t &regel);
+	void bewerte_haus(koord pos, sint32 rd, rule_t &regel);
 
 	void pruefe_grenzen(koord pos);
 
@@ -338,13 +357,19 @@ public:
 	 * gibt das pax-statistik-array fr letzten monat zurck
 	 * @author Hj. Malthaner
 	 */
-	const array2d_tpl<unsigned char>* get_pax_ziele_alt() const { return &pax_ziele_alt; }
+	const sparse_tpl<unsigned char>* get_pax_destinations_old() const { return &pax_destinations_old; }
 
 	/**
 	 * gibt das pax-statistik-array fr den aktuellen monat zurck
 	 * @author Hj. Malthaner
 	 */
-	const array2d_tpl<unsigned char>* get_pax_ziele_neu() const { return &pax_ziele_neu; }
+	const sparse_tpl<unsigned char>* get_pax_destinations_new() const { return &pax_destinations_new; }
+
+	/* this counter will increment by one for every change
+	 * => dialogs can question, if they need to update map
+	 * @author prissi
+	 */
+	unsigned long get_pax_destinations_new_change() const { return pax_destinations_new_change; }
 
 	/**
 	 * Erzeugt eine neue Stadt auf Planquadrat (x,y) die dem Spieler sp
@@ -393,6 +418,10 @@ public:
 	* @author prissi */
 	void change_size( long delta_citicens );
 
+	// when ng is false, no town growth any more
+	void set_citygrowth_yesno( bool ng ) { allow_citygrowth = ng; }
+	bool get_citygrowth() const { return allow_citygrowth; }
+
 	void step(long delta_t);
 
 	void neuer_monat();
@@ -409,6 +438,7 @@ public:
 	 * @author Hj. Malthaner
 	 */
 	inline koord get_pos() const {return pos;}
+	inline koord get_townhall_road() const {return townhall_road;}
 
 	inline koord get_linksoben() const { return lo;}
 	inline koord get_rechtsunten() const { return ur;}
