@@ -11,7 +11,6 @@
 
 /* Flag for Intel byte order
  * SET THIS IN YOUR COMPILER COMMAND LINE!
- * big endian also not completely supported yet
  */
 //#define LITTLE_ENDIAN
 
@@ -24,12 +23,12 @@
 /* single height is only 8 pixel (default 16) */
 //#define HALF_HEIGHT
 
-/* normal ground can be covered by other ground (building does not work correctly yet ...) */
-//#define COVER_TILES
-
-/* construct automatic bridges also a active player */
+/* construct automatic bridges also as active player */
 //#define AUTOMATIC_BRIDGES
 
+/* construct automatic tunnels also as active player */
+//#define AUTOMATIC_TUNNELS
+
 /* citycars have a destination; if they could get near, they dissolve */
 //#define DESTINATION_CITYCARS
 
@@ -40,8 +39,8 @@
  * needed i.e. for MSVC and PowerPC */
 //#define USE_C
 
-// maximum distance to look ahead for tiles (if undefined, uses all the route, recommended)
-//#define MAX_CHOOSE_BLOCK_TILES (64)
+// maximum distance to look ahead for tiles (if undefined, it is unlimited)
+#define MAX_CHOOSE_BLOCK_TILES (64)
 
 // The wind (i.e. approach direction) is random all over the map (not recommended, since it confuses players)
 //#define USE_DIFFERENT_WIND
@@ -52,58 +51,31 @@
 // define this for automaticcally joining stations next to a public stop with it
 //#define AUTOJOIN_PUBLIC
 
-/******************************* obsolete stuf ******************************/
-
-/* storage shed (old implementation) LIKELY NOT WORKING */
-//#define LAGER_NOT_IN_USE
-
 /*********************** Useful things for debugging ... ********************/
 
 /* will highlite marked areas and convoi will leave traces */
 //#define DEBUG_ROUTES
 
-/* shows which tiles are grown as dings (used in boden/grund.cc) */
-//#define SHOW_FORE_GRUND 1
+/* shows which tiles are drawn as dings (used in boden/grund.cc) */
+//#define SHOW_FORE_GRUND
 
 /* shows with block needed update and which not */
 //#define DEBUG_FLUSH_BUFFER
 
-/* shows reserved tiles */
-//#define DEBUG_PDS
-
 /**************************** automatic stuff ********************************/
 
 
 // inclination types
 #ifndef DOUBLE_GROUNDS
-// single height definitions
-#define SOUTH_SLOPE (12)
-#define NORTH_SLOPE (3)
-#define WEST_SLOPE (6)
-#define EAST_SLOPE (9)
+// constants used in tools wkz_setslope / wkz_restoreslope_t
 #define ALL_UP_SLOPE (16)
 #define ALL_DOWN_SLOPE (17)
 #define RESTORE_SLOPE (18)
-
-#define corner1(i) (i%2)
-#define corner2(i) ((i/2)%2)
-#define corner3(i) ((i/4)%2)
-#define corner4(i) (i/8)
-
 #else
 // double height (two slopes per tile) definitions
-#define SOUTH_SLOPE (36)
-#define NORTH_SLOPE (4)
-#define WEST_SLOPE (12)
-#define EAST_SLOPE (28)
 #define ALL_UP_SLOPE (82)
 #define ALL_DOWN_SLOPE (83)
 #define RESTORE_SLOPE (84)
-
-#define corner1(i) (i%3)
-#define corner2(i) ((i/3)%3)
-#define corner3(i) ((i/9)%9)
-#define corner4(i) (i/27)
 #endif
 
 
@@ -154,12 +126,9 @@
 #define Z_GRID (0)
 
 
-
-
-#ifdef OTTD_LIKE
-#define DEFAULT_OBJPATH "pak.ttd/"
-#else
-#define DEFAULT_OBJPATH "pak/"
+// sanity check: USE_C if not GCC and not intel 32bit
+#if !defined USE_C && (!defined __GNUC__ || !defined __i386__)
+#	define USE_C
 #endif
 
 #endif
