@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997 - 2001 Hansjrg Malthaner
+ * Copyright (c) 1997 - 2001 Hj. Malthaner
  *
  * This file is part of the Simutrans project under the artistic licence.
  * (see licence.txt)
@@ -36,21 +36,14 @@ extern log_t *dbg;
  * Inits logging facility.
  * @author Hj. Malthaner
  */
-void init_logging(const char *logname, bool force_flush, bool log_debug);
-
-
-/**
- * Console output, realised as a macro
- * @author Hj. Malthaner
- */
-
-#define print printf
+void init_logging(const char *logname, bool force_flush, bool log_debug, const char *greeting );
 
 #ifndef DEBUG
 
 // nothing to debug -> then ignore
 #define DBG_MESSAGE(i,...) ;
 #define DBG_DEBUG(i,...) ;
+#define DBG_DEBUG4(i,...) ;
 
 #else
 
@@ -58,6 +51,7 @@ void init_logging(const char *logname, bool force_flush, bool log_debug);
 //#define DBG_DEBUG(i,...) dbg->message(i,__VA_ARGS__)
 #define DBG_MESSAGE dbg->message
 #define DBG_DEBUG dbg->message
+#define DBG_DEBUG4 dbg->debug
 
 #endif
 
