@@ -10,20 +10,23 @@
 
 #include "convoihandle_t.h"
 #include "linehandle_t.h"
+#include "halthandle_t.h"
 
-#include "simdebug.h"
-#include "simtypes.h"
 #include "simdings.h"
+#include "simtypes.h"
 
 #include "bauer/warenbauer.h"
 
-#include "dataobj/koord3d.h"
+#include "besch/ware_besch.h"
+
+#include "dataobj/koord.h"
 
 #include "tpl/slist_tpl.h"
 #include "tpl/vector_tpl.h"
-#include "tpl/array_tpl.h"
 
 
+#define RESCHEDULING (1)
+#define REROUTING (2)
 
 #define MAX_HALT_COST   7 // Total number of cost items
 #define MAX_MONTHS     12 // Max history
@@ -36,18 +39,18 @@
 #define HALT_NOROUTE         5 // number of no-route passangers
 #define HALT_CONVOIS_ARRIVED             6 // number of convois arrived this month
 
-class spieler_t;
+class cbuffer_t;
+class grund_t;
 class fabrik_t;
 class karte_t;
+class koord3d;
 #ifdef LAGER_NOT_IN_USE
 class lagerhaus_t;
 #endif
-class ware_t;
-class grund_t;
+class loadsave_t;
 class schedule_t;
-class cbuffer_t;
-
-#include "halthandle_t.h"
+class spieler_t;
+class ware_t;
 
 // -------------------------- Haltestelle ----------------------------
 
@@ -65,6 +68,8 @@ class haltestelle_t
 public:
 	enum station_flags { NOT_ENABLED=0, PAX=1, POST=2, WARE=4, CROWDED=8 };
 
+	enum routine_result_flags { NO_ROUTE=0, ROUTE_OK=1, ROUTE_OVERCROWDED=8 };
+
 	//13-Jan-02     Markus Weber    Added
 	enum stationtyp {invalid=0, loadingbay=1, railstation = 2, dock = 4, busstop = 8, airstop = 16, monorailstop = 32, tramstop = 64, maglevstop=128, narrowgaugestop=256 }; //could be combined with or!
 
@@ -95,11 +100,40 @@ private:
 	void init_financial_history();
 
 	uint8 status_color;
-	uint16 capacity[3]; // passenger, post, goods
+	uint32 capacity[3]; // passenger, post, goods
 	uint8 overcrowded[8];	// bit set, when overcrowded
-	void recalc_status();
+
+	static uint8 status_step;	// NONE or SCHEDULING or REROUTING
+
+	/**
+	 * Markers used in suche_route() to avoid processing the same halt more than once
+	 * Originally they are instance variables of haltestelle_t
+	 * Now consolidated into a static array to speed up suche_route()
+	 * @author Knightly
+	 */
+	static uint8 markers[65536];
+	static uint8 current_mark;
+
+	slist_tpl<convoihandle_t> loading_here;
+	long last_loading_step;
 
 public:
+	// add convoi to loading queue
+	void request_loading( convoihandle_t cnv );
+
+	// removes convoi from laoding quee
+	void finish_loading( convoihandle_t cnv ) { loading_here.remove(cnv); }
+
+	/* recalculates the station bar */
+	void recalc_status();
+
+	/**
+	 * Handles changes of schedules and the resulting rerouting
+	 */
+	static void step_all();
+
+	static uint8 get_rerouting_status() { return status_step; }
+
 	/**
 	 * Tries to generate some pedestrians on the sqaure and the
 	 * adjacent sqaures. Return actual number of generated
@@ -107,18 +141,18 @@ public:
 	 *
 	 * @author Hj. Malthaner
 	 */
-	static int erzeuge_fussgaenger(karte_t *welt, koord3d pos, int anzahl);
+	static int erzeuge_fussgaenger(karte_t *welt, const koord3d pos, int anzahl);
 
 	/* searches for a stop at the given koordinate
-	 * this is called damend often, so we should think about inline it
 	 * @return halthandle_t(), if nothing found
 	 * @author prissi
 	 */
-	static halthandle_t get_halt(karte_t *welt, const koord pos);
+	static halthandle_t get_halt(karte_t *welt, const koord pos, const spieler_t *sp );
 
-	// Hajo: for future compatibility, migrate to this call
-	// but since we allow only for a single stop per planquadrat, this is as good as the above
-	static halthandle_t get_halt(karte_t *welt, const koord3d pos);
+	/* since we allow only for a single stop per planquadrat
+	 * this will always return something even if there is not stop some of the ground level
+	 */
+	static halthandle_t get_halt( karte_t *welt, const koord3d pos, const spieler_t *sp );
 
 	static const slist_tpl<halthandle_t>& get_alle_haltestellen() { return alle_haltestellen; }
 
@@ -175,6 +209,8 @@ public:
 		convoihandle_t reservation;
 	};
 
+	const slist_tpl<tile_t> &get_tiles() const { return tiles; };
+
 private:
 	slist_tpl<tile_t> tiles;
 
@@ -183,7 +219,19 @@ private:
 	// List with all reachable destinations
 	vector_tpl<halthandle_t>* warenziele;
 
-	// loest warte_menge ab
+	/**
+	 * For each schedule/line, that adds halts to a warenziel array,
+	 * this counter is incremented. Each ware category needs a separate
+	 * counter. If this counter is more than 1, this halt is a transfer
+	 * halt, i.e. contains non_identical_schedules with overlapping
+	 * destinations.
+	 * Non-transfer stops do not need to be searched for connections
+	 * => large speedup possible.
+	 * @author Knightly
+	 */
+	uint8 *non_identical_schedules;
+
+	// Array with different categries that contains all waiting goods at this stop
 	vector_tpl<ware_t> **waren;
 
 	/**
@@ -203,6 +251,9 @@ private:
 
 	uint8 rebuilt_destination_counter;	// new schedule, first rebuilt destinations asynchroniously
 	uint8 reroute_counter;						// the reroute goods
+	// since we do partial routing, we remeber the last offset
+	uint8 last_catg_index;
+	uint32 last_ware_index;
 
 	/* station flags (most what enabled) */
 	uint8 enables;
@@ -229,13 +280,6 @@ private:
 	 */
 	uint32 pax_unhappy;
 
-	/**
-	 * Haltestellen werden beim warenrouting markiert. Jeder durchgang
-	 * hat eine eindeutige marke
-	 * @author Hj. Malthaner
-	 */
-	uint32 marke;
-
 #ifdef USE_QUOTE
 	// for station rating
 	const char * quote_bezeichnung(int quote) const;
@@ -256,6 +300,13 @@ private:
 	 */
 	void liefere_an_fabrik(const ware_t& ware);
 
+	/*
+	 * transfers all goods to given station
+	 *
+	 * @author Dwachs
+	 */
+	void transfer_goods(halthandle_t halt);
+
 	/*
 	* parameter to ease sorting
 	* sortierung is local and stores the sortorder for the individual station
@@ -270,11 +321,12 @@ private:
 
 public:
 	/**
-	* Called every 255 steps
+	* Called after schedule calculation of all stations is finished
 	* will distribute the goods to changed routes (if there are any)
+	* returns true upon completion
 	* @author Hj. Malthaner
 	*/
-	void reroute_goods();
+	bool reroute_goods(sint16 &units_remaining);
 
 	/**
 	 * getter/setter for sortby
@@ -305,10 +357,10 @@ public:
 
 	/**
 	 * Rebuilds the list of reachable destinations
-	 *
+	 * returns the search number of connections
 	 * @author Hj. Malthaner
 	 */
-	void rebuild_destinations();
+	sint32 rebuild_destinations();
 
 	uint8 get_rebuild_destination_counter() const  { return rebuilt_destination_counter; }
 
@@ -330,10 +382,10 @@ public:
 	const slist_tpl<fabrik_t*>& get_fab_list() const { return fab_list; }
 
 	/**
-	 * Haltestellen messen regelmaessig die Fahrplaene pruefen
+	 * called regularily to update status and reroute stuff
 	 * @author Hj. Malthaner
 	 */
-	void step();
+	bool step(sint16 &units_remaining);
 
 	/**
 	 * Called every month/every 24 game hours
@@ -341,7 +393,7 @@ public:
 	 */
 	void neuer_monat();
 
-	karte_t* get_welt() const { return welt; }
+	static karte_t* get_welt() { return welt; }
 
 	/**
 	 * Kann die Ware nicht zum Ziel geroutet werden (keine Route), dann werden
@@ -351,15 +403,13 @@ public:
 	 * @author Hj. Malthaner
 	 *
 	 * for reverse routing, also the next to last stop can be added, if next_to_ziel!=NULL
+	 *
+	 * if avoid_overcrowding is set, a valid route in only found when there is no overflowing stop in between
+	 *
 	 * @author prissi
 	 */
-	void suche_route(ware_t &ware, koord *next_to_ziel=NULL);
-#if 0
-	/* true, if there is a conncetion between these places
-	 * @author prissi
-	 */
-	bool is_connected(const halthandle_t halt, const ware_besch_t * wtyp) const { return get_warenziele(wtyp->get_catg_index(halt))->is_contained(); }
-#endif
+	int suche_route( ware_t &ware, koord *next_to_ziel, const bool no_routing_over_overcrowding );
+
 	int get_pax_enabled()  const { return enables & PAX;  }
 	int get_post_enabled() const { return enables & POST; }
 	int get_ware_enabled() const { return enables & WARE; }
@@ -376,6 +426,18 @@ public:
 		return enables&WARE;
 	}
 
+	// a separate version for checking with goods category index
+	int is_enabled( const uint8 ctg )
+	{
+		if (ctg==0) {
+			return enables&PAX;
+		}
+		else if(ctg==1) {
+			return enables&POST;
+		}
+		return enables&WARE;
+	}
+
 	/**
 	 * Found route and station uncrowded
 	 * @author Hj. Malthaner
@@ -404,7 +466,7 @@ public:
 #endif
 
 	bool add_grund(grund_t *gb);
-	void rem_grund(grund_t *gb);
+	bool rem_grund(grund_t *gb);
 
 	uint32 get_capacity(uint8 typ) const { return capacity[typ]; }
 
@@ -440,13 +502,6 @@ public:
 	 */
 	uint32 get_ware_fuer_zwischenziel(const ware_besch_t *warentyp, const halthandle_t zwischenziel) const;
 
-	/**
-	 * @returns the sum of all waiting goods (100t coal + 10
-	 * passengers + 2000 liter oil = 2110)
-	 * @author Markus Weber
-	 */
-	uint32 sum_all_waiting_goods() const;
-
 	// true, if we accept/deliver this kind of good
 	bool gibt_ab(const ware_besch_t *warentyp) const { return waren[warentyp->get_catg_index()] != NULL; }
 
@@ -460,7 +515,7 @@ public:
 	 * @return abgeholte menge
 	 * @author Hj. Malthaner
 	 */
-	ware_t hole_ab(const ware_besch_t *warentyp, uint32 menge, schedule_t *fpl);
+	ware_t hole_ab( const ware_besch_t *warentyp, uint32 menge, const schedule_t *fpl, const spieler_t *sp );
 
 	/* liefert ware an. Falls die Ware zu wartender Ware dazugenommen
 	 * werden kann, kann ware_t gelscht werden! D.h. man darf ware nach
@@ -475,14 +530,6 @@ public:
 	uint32 liefere_an(ware_t ware);
 	uint32 starte_mit_route(ware_t ware);
 
-	/**
-	 * wird von Fahrzeug aufgerufen, wenn dieses an der Haltestelle
-	 * gehalten hat.
-	 * @param typ der befrderte warentyp
-	 * @author Hj. Malthaner
-	 */
-	void hat_gehalten(const ware_besch_t *warentyp, const schedule_t *fpl);
-
 	const grund_t *find_matching_position(waytype_t wt) const;
 
 	/* checks, if there is an unoccupied loading bay for this kind of thing
@@ -528,7 +575,7 @@ public:
 	void zeige_info();
 
 	/**
-	 * @returns the type of a station
+	 * @return the type of a station
 	 * (combination of: railstation, loading bay, dock)
 	 * @author Markus Weber
 	 */
@@ -546,7 +593,7 @@ public:
 	void set_name(const char *name);
 
 	// create an unique name: better to be called with valid handle, althoug it will work without
-	char *create_name(const koord k, const char *typ);
+	char *create_name(const koord k, const char *typ, const int lang);
 
 	void rdwr(loadsave_t *file);
 
@@ -570,6 +617,24 @@ public:
 	 */
 	vector_tpl<linehandle_t> registered_lines;
 
+	/**
+	 * Register a lineless convoy which serves this stop
+	 * @author Knightly
+	 */
+	void add_convoy(convoihandle_t convoy) { registered_convoys.append_unique(convoy); }
+
+	/**
+	 * Unregister a lineless convoy
+	 * @author Knightly
+	 */
+	void remove_convoy(convoihandle_t convoy) { registered_convoys.remove(convoy); }
+
+	/**
+	 * A list of lineless convoys serving this stop
+	 * @author Knightly
+	 */
+	vector_tpl<convoihandle_t> registered_convoys;
+
 	/**
 	 * book a certain amount into the halt's financial history
 	 * @author hsiegeln
@@ -586,7 +651,7 @@ public:
 	 * return a specified element from the financial history
 	 * @author hsiegeln
 	 */
-	sint64 get_finance_history(int month, int cost_type) { return financial_history[month][cost_type]; }
+	sint64 get_finance_history(int month, int cost_type) const { return financial_history[month][cost_type]; }
 
 	// flags station for a crowded message at the beginning of next month
 	void bescheid_station_voll() { enables |= CROWDED; status_color = COL_RED; }
@@ -595,5 +660,17 @@ public:
 	* @author prissi
 	*/
 	void mark_unmark_coverage(const bool mark) const;
+
+	/*
+	* deletes factory references so map rotation won't segfault
+	*/
+	void release_factory_links();
+
+	/**
+	 * Initialise the markers to zero
+	 * @author Knightly
+	 */
+	static void init_markers();
+
 };
 #endif
