@@ -242,17 +242,25 @@ void modal_dialogue( gui_frame_t *gui, long magic, karte_t *welt, bool (*quit)()
 			// do not move, do not close it!
 			dr_prepare_flush();
 			gui->zeichnen( koord(win_get_posx(gui),win_get_posy(gui)), gui->get_fenstergroesse() );
-			display_poll_event(&ev);
-			// main window resized
-			check_pos_win(&ev);
 			dr_flush();
-			dr_sleep(50);
-			// main window resized
-			if(ev.ev_class==EVENT_SYSTEM  &&  ev.ev_code==SYSTEM_RESIZE) {
-				// main window resized
-				simgraph_resize( ev.mx, ev.my );
-				display_fillbox_wh( 0, 0, ev.mx, ev.my, COL_BLACK, true );
+
+			display_poll_event(&ev);
+			if(ev.ev_class==EVENT_SYSTEM) {
+				if (ev.ev_code==SYSTEM_RESIZE) {
+					// main window resized
+					simgraph_resize( ev.mx, ev.my );
+					display_fillbox_wh( 0, 0, ev.mx, ev.my, COL_BLACK, true );
+				}
+				else if (ev.ev_code == SYSTEM_QUIT) {
+					umgebung_t::quit_simutrans = true;
+					break;
+				}
 			}
+			else {
+				// other events
+				check_pos_win(&ev);
+			}
+			dr_sleep(50);
 		}
 		set_pointer(1);
 		display_fillbox_wh( 0, 0, display_get_width(), display_get_height(), COL_BLACK, true );
@@ -679,6 +687,10 @@ int simu_main(int argc, char** argv)
 	if(  umgebung_t::objfilename.empty()  ) {
 		show_pointer(1);
 		ask_objfilename();
+		if(  umgebung_t::quit_simutrans  ) {
+			simgraph_exit();
+			return 0;
+		}
 		if(  umgebung_t::objfilename.empty()  ) {
 			// nothing to be loaded => exit
 			fprintf(stderr, "*** No pak set found ***\n\nMost likely, you have no pak set installed.\nPlease download and install also graphics (pak).\n");
